Package: detectseparation
Title: Detect and Check for Separation and Infinite Maximum Likelihood
        Estimates
Version: 0.4.0
Authors@R: c(person(given = "Ioannis", family = "Kosmidis", role = c("aut", "cre"), email = "ioannis.kosmidis@warwick.ac.uk", comment = c(ORCID = "0000-0003-1556-0302")),
             person(given = "Dirk", family = "Schumacher", role = "aut", email = "mail@dirk-schumacher.net"),
             person(given = "Florian", family = "Schwendinger", role = "aut", email = "FlorianSchwendinger@gmx.at"),
             person(given = "Kjell", family = "Konis", role = "ctb", email = "kjell.konis@me.com"))
Description: Provides pre-fit and post-fit methods for detecting separation and infinite maximum likelihood estimates in generalized linear models with categorical responses. The pre-fit methods apply on binomial-response generalized liner models such as logit, probit and cloglog regression, and can be directly supplied as fitting methods to the glm() function. They solve the linear programming problems for the detection of separation developed in Konis (2007, <https://ora.ox.ac.uk/objects/uuid:8f9ee0d0-d78e-4101-9ab4-f9cbceed2a2a>) using 'ROI' <https://cran.r-project.org/package=ROI> or 'lpSolveAPI' <https://cran.r-project.org/package=lpSolveAPI>. The post-fit methods apply to models with categorical responses, including binomial-response generalized linear models and multinomial-response models, such as baseline category logits and adjacent category logits models; for example, the models implemented in the 'brglm2' <https://cran.r-project.org/package=brglm2> package. The post-fit methods successively refit the model with increasing number of iteratively reweighted least squares iterations, and monitor the ratio of the estimated standard error for each parameter to what it has been in the first iteration. According to the results in Lesaffre & Albert (1989, <https://www.jstor.org/stable/2345845>), divergence of those ratios indicates data separation.
URL: https://github.com/ikosmidis/detectseparation
BugReports: https://github.com/ikosmidis/detectseparation/issues
Imports: ROI, ROI.plugin.lpsolve, lpSolveAPI, pkgload
Depends: R (>= 3.3.0)
License: GPL-3
Encoding: UTF-8
LazyData: true
RoxygenNote: 7.3.3
Suggests: knitr, rmarkdown, covr, brglm2, AER, tinytest,
        ROI.plugin.ecos, ROI.plugin.glpk, ROI.plugin.alabama,
        ROI.plugin.neos
VignetteBuilder: knitr
NeedsCompilation: no
Packaged: 2026-04-17 07:32:46 UTC; yiannis
Author: Ioannis Kosmidis [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-1556-0302>),
  Dirk Schumacher [aut],
  Florian Schwendinger [aut],
  Kjell Konis [ctb]
Maintainer: Ioannis Kosmidis <ioannis.kosmidis@warwick.ac.uk>
Repository: CRAN
Date/Publication: 2026-04-19 05:10:24 UTC
Built: R 4.5.2; ; 2026-04-19 05:34:24 UTC; unix

Package: detectseparation
Title: Detect and Check for Separation and Infinite Maximum Likelihood
        Estimates
Version: 0.4.0
Authors@R: c(person(given = "Ioannis", family = "Kosmidis", role = c("aut", "cre"), email = "ioannis.kosmidis@warwick.ac.uk", comment = c(ORCID = "0000-0003-1556-0302")),
             person(given = "Dirk", family = "Schumacher", role = "aut", email = "mail@dirk-schumacher.net"),
             person(given = "Florian", family = "Schwendinger", role = "aut", email = "FlorianSchwendinger@gmx.at"),
             person(given = "Kjell", family = "Konis", role = "ctb", email = "kjell.konis@me.com"))
Description: Provides pre-fit and post-fit methods for detecting separation and infinite maximum likelihood estimates in generalized linear models with categorical responses. The pre-fit methods apply on binomial-response generalized liner models such as logit, probit and cloglog regression, and can be directly supplied as fitting methods to the glm() function. They solve the linear programming problems for the detection of separation developed in Konis (2007, <https://ora.ox.ac.uk/objects/uuid:8f9ee0d0-d78e-4101-9ab4-f9cbceed2a2a>) using 'ROI' <https://cran.r-project.org/package=ROI> or 'lpSolveAPI' <https://cran.r-project.org/package=lpSolveAPI>. The post-fit methods apply to models with categorical responses, including binomial-response generalized linear models and multinomial-response models, such as baseline category logits and adjacent category logits models; for example, the models implemented in the 'brglm2' <https://cran.r-project.org/package=brglm2> package. The post-fit methods successively refit the model with increasing number of iteratively reweighted least squares iterations, and monitor the ratio of the estimated standard error for each parameter to what it has been in the first iteration. According to the results in Lesaffre & Albert (1989, <https://www.jstor.org/stable/2345845>), divergence of those ratios indicates data separation.
URL: https://github.com/ikosmidis/detectseparation
BugReports: https://github.com/ikosmidis/detectseparation/issues
Imports: ROI, ROI.plugin.lpsolve, lpSolveAPI, pkgload
Depends: R (>= 3.3.0)
License: GPL-3
Encoding: UTF-8
LazyData: true
RoxygenNote: 7.3.3
Suggests: knitr, rmarkdown, covr, brglm2, AER, tinytest,
        ROI.plugin.ecos, ROI.plugin.glpk, ROI.plugin.alabama,
        ROI.plugin.neos
VignetteBuilder: knitr
NeedsCompilation: no
Packaged: 2026-04-17 07:32:46 UTC; yiannis
Author: Ioannis Kosmidis [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-1556-0302>),
  Dirk Schumacher [aut],
  Florian Schwendinger [aut],
  Kjell Konis [ctb]
Maintainer: Ioannis Kosmidis <ioannis.kosmidis@warwick.ac.uk>
Repository: CRAN
Date/Publication: 2026-04-19 05:10:24 UTC
Built: R 4.6.0; ; 2026-04-26 12:26:01 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(check_infinite_estimates,glm)
S3method(plot,inf_check)
S3method(print,detect_infinite_estimates)
S3method(print,detect_separation)
export(checkInfiniteEstimates)
export(check_infinite_estimates)
export(detectInfiniteEstimates)
export(detectSeparation)
export(detectSeparationControl)
export(detect_infinite_estimates)
export(detect_separation)
export(detect_separation_control)
import(ROI.plugin.lpsolve)
importFrom(graphics,matplot)
importFrom(stats,binomial)
importFrom(stats,coef)
importFrom(stats,coefficients)
importFrom(stats,gaussian)
importFrom(stats,nobs)
importFrom(stats,sd)
importFrom(stats,update)
importFrom(stats,vcov)

# detectseparation <img src="man/figures/hex_detectseparation.svg" width="320" align="right">

<!-- badges: start -->

[![CRAN
status](https://www.r-pkg.org/badges/version/detectseparation)](https://CRAN.R-project.org/package=detectseparation)
[![Licence](https://img.shields.io/badge/licence-GPL--3-blue.svg)](https://www.gnu.org/licenses/gpl-3.0.en.html)
[![R-CMD-check](https://github.com/ikosmidis/detectseparation/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/ikosmidis/detectseparation/actions/workflows/R-CMD-check.yaml)
[![Codecov test
coverage](https://codecov.io/gh/ikosmidis/detectseparation/graph/badge.svg)](https://app.codecov.io/gh/ikosmidis/detectseparation)
<!-- badges: end -->

[**detectseparation**](https://github.com/ikosmidis/detectseparation)
provides *pre-fit* and *post-fit* methods for the detection of
separation and of infinite maximum likelihood estimates in binomial
response generalized linear models.

The key methods are `detect_infinite_estimates()`, `detect_separation()`
and `check_infinite_estimates()`.

## Installation

You can install the released version of detectseparation from
[CRAN](https://CRAN.R-project.org) with:

    install.packages("detectseparation")

and the development version from [GitHub](https://github.com/) with:

    # install.packages("devtools")
    devtools::install_github("ikosmidis/detectseparation", ref = "develop")

## Detecting and checking for Infinite maximum likelihood estimates

Heinze and Schemper (2002) used a logistic regression model to analyze
data from a study on endometrial cancer (see, Agresti 2015, sec. 5.7 or
`?endometrial` for more details on the data set). Below, we refit the
model in Heinze and Schemper (2002) in order to demonstrate the
functionality that **detectseparation** provides.

    library("detectseparation")
    data("endometrial", package = "detectseparation")
    endo_glm <- glm(HG ~ NV + PI + EH, family = binomial(), data = endometrial)
    theta_mle <- coef(endo_glm)
    summary(endo_glm)
    #> 
    #> Call:
    #> glm(formula = HG ~ NV + PI + EH, family = binomial(), data = endometrial)
    #> 
    #> Coefficients:
    #>               Estimate Std. Error z value Pr(>|z|)    
    #> (Intercept)    4.30452    1.63730   2.629 0.008563 ** 
    #> NV            18.18556 1715.75089   0.011 0.991543    
    #> PI            -0.04218    0.04433  -0.952 0.341333    
    #> EH            -2.90261    0.84555  -3.433 0.000597 ***
    #> ---
    #> Signif. codes:  0 '***' 0.001 '**' 0.01 '*' 0.05 '.' 0.1 ' ' 1
    #> 
    #> (Dispersion parameter for binomial family taken to be 1)
    #> 
    #>     Null deviance: 104.903  on 78  degrees of freedom
    #> Residual deviance:  55.393  on 75  degrees of freedom
    #> AIC: 63.393
    #> 
    #> Number of Fisher Scoring iterations: 17

The maximum likelihood (ML) estimate of the parameter for `NV` is
actually infinite. The reported, apparently finite value is merely due
to false convergence of the iterative estimation procedure. The same is
true for the estimated standard error, and, hence the value 0.011 for
the *z*-statistic cannot be trusted for inference on the size of the
effect for `NV`.

### `detect_separation()`

`detect_separation()` is a *pre-fit* method, in the sense that it does
not need to estimate the model to detect separation and/or identify
infinite estimates. For example,

    endo_sep <- glm(HG ~ NV + PI + EH, data = endometrial,
                    family = binomial("logit"),
                    method = "detect_separation")
    endo_sep
    #> Implementation: ROI | Solver: lpsolve 
    #> Separation: TRUE 
    #> Existence of maximum likelihood estimates
    #> (Intercept)          NV          PI          EH 
    #>           0         Inf           0           0 
    #> 0: finite value, Inf: infinity, -Inf: -infinity

So, the actual maximum likelihood estimates are

    coef(endo_glm) + coef(endo_sep)
    #> (Intercept)          NV          PI          EH 
    #>   4.3045178         Inf  -0.0421834  -2.9026056

and the estimated standard errors are

    coef(summary(endo_glm))[, "Std. Error"] + abs(coef(endo_sep))
    #> (Intercept)          NV          PI          EH 
    #>  1.63729861         Inf  0.04433196  0.84555156

We can ask `detect_separation()` not only to detect separation, but
also, if separation is detected, to solve an additional linear program
to check whether separation is complete or quasi-complete. We do this by
setting `separation_type = TRUE` in the `glm()` call

    glm(HG ~ NV + PI + EH, data = endometrial, family = binomial("logit"),
        method = "detect_separation", separation_type = TRUE)
    #> Implementation: ROI | Solver: lpsolve 
    #> Separation: TRUE (quasi-complete)
    #> Existence of maximum likelihood estimates
    #> (Intercept)          NV          PI          EH 
    #>           0         Inf           0           0 
    #> 0: finite value, Inf: infinity, -Inf: -infinity

We can of course, simply, use the update method on the `glm` object, if
that is available, and change the `method` argument with any extra
options. For example,

    update(endo_glm, method = "detect_separation", separation_type = TRUE)
    #> Implementation: ROI | Solver: lpsolve 
    #> Separation: TRUE (quasi-complete)
    #> Existence of maximum likelihood estimates
    #> (Intercept)          NV          PI          EH 
    #>           0         Inf           0           0 
    #> 0: finite value, Inf: infinity, -Inf: -infinity

### `check_infinite_estimates()`

Lesaffre and Albert (1989, sec. 4) describe a procedure that can hint on
the occurrence of infinite estimates. In particular, the model is
successively refitted, by increasing the maximum number of allowed
iteratively re-weighted least squares iterations at each step. The
estimated asymptotic standard errors from each step are, then, divided
to the corresponding ones from the first fit. If the sequence of ratios
diverges, then the maximum likelihood estimate of the corresponding
parameter is minus or plus infinity. The following code chunk applies
this process to `endo_glm`.

    (inf_check <- check_infinite_estimates(endo_glm))
    #>       (Intercept)           NV       PI       EH
    #>  [1,]    1.000000 1.000000e+00 1.000000 1.000000
    #>  [2,]    1.424352 2.092407e+00 1.466885 1.672979
    #>  [3,]    1.590802 8.822303e+00 1.648003 1.863563
    #>  [4,]    1.592818 6.494231e+01 1.652508 1.864476
    #>  [5,]    1.592855 7.911035e+02 1.652591 1.864492
    #>  [6,]    1.592855 1.588973e+04 1.652592 1.864493
    #>  [7,]    1.592855 5.298760e+05 1.652592 1.864493
    #>  [8,]    1.592855 2.332822e+07 1.652592 1.864493
    #>  [9,]    1.592855 2.332822e+07 1.652592 1.864493
    #> [10,]    1.592855 2.332822e+07 1.652592 1.864493
    #> [11,]    1.592855 2.332822e+07 1.652592 1.864493
    #> [12,]    1.592855 2.332822e+07 1.652592 1.864493
    #> [13,]    1.592855 2.332822e+07 1.652592 1.864493
    #> [14,]    1.592855 2.332822e+07 1.652592 1.864493
    #> [15,]    1.592855 2.332822e+07 1.652592 1.864493
    #> [16,]    1.592855 2.332822e+07 1.652592 1.864493
    #> [17,]    1.592855 2.332822e+07 1.652592 1.864493
    #> [18,]    1.592855 2.332822e+07 1.652592 1.864493
    #> [19,]    1.592855 2.332822e+07 1.652592 1.864493
    #> [20,]    1.592855 2.332822e+07 1.652592 1.864493
    #> attr(,"class")
    #> [1] "inf_check"
    plot(inf_check)

<img src="man/figures/README-unnamed-chunk-7-1.png" width="100%" />

# References

Agresti, A. 2015. *Foundations of Linear and Generalized Linear Models*.
Wiley Series in Probability and Statistics. Wiley.

Heinze, G., and M. Schemper. 2002. “A Solution to the Problem of
Separation in Logistic Regression.” *Statistics in Medicine* 21:
2409–19.

Lesaffre, E., and A. Albert. 1989. “Partial Separation in Logistic
Discrimination.” *Journal of the Royal Statistical Society. Series B
(Methodological)* 51 (1): 109–16.
<https://www.jstor.org/stable/2345845>.

## endometrial data from Heinze \& Schemper (2002) (see ?endometrial)
data("endometrial", package = "detectseparation")

expect_message(endometrial_separation <- glm(HG ~ I(-NV) + PI + EH, data = endometrial,
                                             family = binomial("log"),
                                             method = "detect_separation"), pattern = "not necessarily")


expect_warning(endometrial_separation <- glm(HG ~ I(-NV) + PI + EH, data = endometrial,
                                             family = binomial("identity"),
                                             method = "detect_separation"), pattern = "reliable")


expect_warning(endometrial_separation <- glm(HG ~ I(-NV) + PI + EH, data = endometrial,
                                             family = poisson("log"),
                                             method = "detect_separation"), pattern = "for use with binomial-response")

# Copyright (C) 2022- Florian Schwendinger, Ioannis Kosmidis

#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 or 3 of the License
#  (at your option).
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  A copy of the GNU General Public License is available at
#  http://www.r-project.org/Licenses/

#' @title Detect Infinite Estimates
#'
#' @description
#'
#' Method for [glm()] that detects infinite components in
#' the maximum likelihood estimates of generalized linear models
#' with binomial responses.
#'
#' @inheritParams stats::glm.fit
#'
#' @aliases detectInfiniteEstimates
#'
#' @param x `x` is a design matrix of dimension `n * p`.
#' @param y `y` is a vector of observations of length `n`.
#' @param control a list of parameters controlling separation
#'     detection. See [detect_separation_control()] for
#'     details.
#' @param start currently not used.
#' @param mustart currently not used.
#' @param etastart currently not used.
#' @param singular.ok logical. If `FALSE`, a singular model is an
#'     error.
#'
#'
#' @details
#'
#' For binomial-response generalized linear models with `"log"` link,
#' separated data allocations do not necessarily lead to infinite
#' maximum likelihood estimates. For this reason, for models with the
#' `"log"` link [detect_infinite_estimates()] relies on an alternative
#' linear optimization model developed in Schwendinger et
#' al. (2021). For all the other supported links
#' [detect_infinite_estimates()] relies on the linear programming
#' methods developed in Konis (2007). See [detect_separation()] for
#' definitions and details.
#'
#' [detect_infinite_estimates()] is a wrapper to the
#' functions `separator_ROI()`, `separator_lpSolveAPI()` (a
#' modified version of the `separator()` function from the
#' \pkg{safeBinaryRegression} R package), and `dielb_ROI()`. The first
#' two are used for non-log links, while `dielb_ROI()` is used for the
#' `"log"` link.
#'
#' The [coefficients()] method extracts a vector of values
#' for each of the model parameters under the following convention:
#' `0` if the maximum likelihood estimate of the parameter is
#' finite, and `Inf` or `-Inf` if the maximum likelihood
#' estimate of the parameter is plus or minus infinity. This
#' convention makes it easy to adjust the maximum likelihood estimates
#' to their actual values by element-wise addition.
#'
#' [detect_infinite_estimates()] can be passed directly as
#' a method to the [glm()] function. See, examples.
#'
#' [detectInfiniteEstimates()] is an alias for
#' [detect_infinite_estimates()].
#'
#' @author Ioannis Kosmidis \[aut, cre\] \email{ioannis.kosmidis@warwick.ac.uk}, Florian Schwendinger \[aut\] \email{FlorianSchwendinger@gmx.at}, Dirk Schumacher \[aut\] \email{mail@dirk-schumacher.net}, Kjell Konis \[ctb\] \email{kjell.konis@me.com}
#'
#' @seealso [glm()], [detect_separation()], [check_infinite_estimates()], [brglm2::brglm_fit()]
#'
#' @references
#'
#' Silvapulle, M. J. (1981).
#' On the Existence of Maximum Likelihood Estimators for the Binomial Response Models.
#' Journal of the Royal Statistical Society. Series B (Methodological), 43(3), 310–313.
#' <https://www.jstor.org/stable/2984941>
#'
#' Konis K. (2007). *Linear Programming Algorithms for Detecting
#' Separated Data in Binary Logistic Regression
#' Models*. DPhil. University of Oxford.
#' <https://ora.ox.ac.uk/objects/uuid:8f9ee0d0-d78e-4101-9ab4-f9cbceed2a2a>
#'
#' Konis K. (2013). safeBinaryRegression: Safe Binary Regression. R
#' package version 0.1-3.
#' <https://CRAN.R-project.org/package=safeBinaryRegression>
#'
#' Kosmidis I. and Firth D. (2021). Jeffreys-prior penalty, finiteness
#' and shrinkage in binomial-response generalized linear
#' models. *Biometrika*, **108**, 71–82. \doi{10.1093/biomet/asaa052}
#'
#' Schwendinger, F., Grün, B. & Hornik, K. (2021). A comparison of
#' optimization solvers for log binomial regression including conic
#' programming.  *Computational Statistics*, **36**,
#' 1721–1754. \doi{10.1007/s00180-021-01084-5}
#'
#'
#' @examples
#' # The classical example given in Silvapulle (1981) can be utilized
#' # to show that for the log-binomial model there exist data allocations
#' # which are separated but produce finite estimates.
#' data("silvapulle1981", package = "detectseparation")
#'
#' # Since the data is separated the MLE does not exist for the logit link.
#' glm(y ~ ghqs, data = silvapulle1981, family = binomial(),
#'     method = "detect_infinite_estimates")
#'
#' # However, for the log link all components of the MLE are finite.
#' glm(y ~ ghqs, data = silvapulle1981, family = binomial("log"),
#'     method = "detect_infinite_estimates")
#' # The maximum likelihood fit is
#' glm(y ~ ghqs, data = silvapulle1981, family = binomial("log"), start = c(-1, 0))
#'
#' @export
detect_infinite_estimates <- function(x, y, weights = NULL,
                                      start = NULL, etastart = NULL,  mustart = NULL,
                                      offset = NULL, family = gaussian(),
                                      control = list(), intercept = TRUE, singular.ok = TRUE) {
    if (isTRUE(family$family != "binomial")) {
        warning("`detect_infinite_estimates()` has been developed for use with binomial-response GLMs")
    }
    nobs <- NROW(y)
    if (is.null(weights)) {
        weights <- rep.int(1, nobs)
    }
    if (is.null(offset)) {
        offset <- rep.int(0, nobs)
    }
    out <- .detect_infinite_estimates(x = x, y = y, weights = weights, start = start,
                                      etastart = etastart,  mustart = mustart,
                                      offset = offset, family = family, control = control,
                                      intercept = control, singular.ok = singular.ok,
                                      log_link = isTRUE(family$link == "log"))
    class(out) <- out$class <- "detect_infinite_estimates"
    out
}

#' @method print detect_infinite_estimates
#' @export
print.detect_infinite_estimates <- function(x, digits = max(5L, getOption("digits") - 3L), ...) {
    cat("Implementation:", x$control$implementation, "| ")
    if (identical(x$control$implementation, "ROI")) {
        cat("Solver:", x$control$solver, "\n")
    }
    else {
        cat("Linear program:", x$control$linear_program, "| Purpose:", x$control$purpose, "\n")
    }
    cat("Infinite estimates:", x$outcome, "\n")
    if (!is.null(x$coefficients)) {
        cat("Existence of maximum likelihood estimates\n")
        print(coefficients(x))
        cat("0: finite value, Inf: infinity, -Inf: -infinity\n")
    }
}

## Workhorse function
.detect_infinite_estimates <- function(x, y, weights = NULL,
                                       start = NULL, etastart = NULL,  mustart = NULL,
                                       offset = NULL, family = gaussian(),
                                       control = list(), intercept = TRUE, singular.ok = TRUE,
                                       log_link = FALSE) {
    control <- do.call("detect_separation_control", control)
    lp <- if (log_link) {
        dielb_ROI
    } else {
        getNamespace("detectseparation")[[paste("separator", control$implementation, sep = "_")]]
    }
    # ensure x is a matrix
    x <- as.matrix(x)
    betas_names_all <- betas_names <- if (is.null(colnames(x))) make.names(seq_len(NCOL(x))) else colnames(x)
    #
    nobs <- NROW(y)
    nvars <- ncol(x)
    if (nvars == 0) {
        return(list(outcome = FALSE, control = control))
    }
    if (is.null(weights)) {
        weights <- rep.int(1, nobs)
    }
    if (missingOffset <- is.null(offset)) {
        offset <- rep.int(0, nobs)
    }
    # Initialize as prescribed in family
    eval(family$initialize)
    if (control$solver == "alabama" & is.null(control$solver_control$start)) {
        control$solver_control$start <- rep(0, nvars)
    }
    # Detect aliasing
    qrx <- qr(x)
    rank <- qrx$rank
    is_full_rank <- rank == nvars
    if (!singular.ok && !is_full_rank) {
        stop("singular fit encountered")
    }
    if (!isTRUE(is_full_rank)) {
        aliased <- qrx$pivot[seq.int(qrx$rank + 1, nvars)]
        X_all <- x
        x <- x[, -aliased]
        betas_names <- betas_names[-aliased]
    }
    betas_all <- structure(rep(NA_real_, length(betas_names_all)), .Names = betas_names_all)
    # Observations with zero weight do not enter calculations so ignore
    keep <- weights > 0
    x <- x[keep, , drop = FALSE]
    y <- y[keep]
    # Reshape data set: keep 0 and 1, and replace anything in (0,
    # 1) with one zero and one 1
    ones <- y == 1
    zeros <- y == 0
    non_boundary <- !(ones | zeros)
    x <- x[c(which(ones), which(zeros), rep(which(non_boundary), 2)), , drop = FALSE]
    y <- c(y[ones], y[zeros], rep(c(0., 1.), each = sum(non_boundary)))
    # Run linear program
    out <- lp(x = x, y = y,
              linear_program = control$linear_program,
              purpose = control$purpose,
              tolerance = control$tolerance,
              solver = control$solver,
              solver_control = control$solver_control)
    if (is.na(out$outcome)) {
        if (identical(control$implementation, "ROI")) {
            warning("unexpected result from implementation ", control$implementation, " with solver: ", control$solver, "\n")
        }
        if (identical(control$implementation, "lsSolveAPI") & !log_link) {
            warning("unexpected result from implementation ", control$implementation, " with linear_program: ", control$linear_program, " and purpose: ", control$purpose, "\n")
        }
    }
    if (is.null(out$beta)) {
        betas_all <- NULL
    }
    else {
        betas <- out$beta
        names(betas) <- betas_names
        inds <- abs(betas) < control$tolerance
        betas <- Inf * betas
        betas[inds] <- 0
        betas_all[betas_names] <- betas[betas_names]
    }
    out <- list(x = x,
                y = y,
                coefficients = betas_all,
                outcome = out$outcome,
                control = control)
    return(out)
}

# ensure the solver is loaded using the ROI plugin mechanism; if
# the plugin is not available then offer to install it
check_ROI_solver <- function(solver) {
    if (solver != "lpsolve") {
        roi_plugin_name <- paste0("ROI.plugin.", solver)
        pkgload::check_suggested(roi_plugin_name, path = pkgload::inst("detectseparation"), version = "*")
        requireNamespace(roi_plugin_name, quietly = TRUE)
    }
}

# Copyright (C) 2016- Ioannis Kosmidis

#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 or 3 of the License
#  (at your option).
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  A copy of the GNU General Public License is available at
#  http://www.r-project.org/Licenses/


#' A simple diagnostic of whether the maximum likelihood estimates are
#' infinite
#'
#'
#' @param object the result of a [glm()] call.
#' @param nsteps starting from `maxit = 1`, the GLM is refitted
#'     for `maxit = 2`, `maxit = 3`, \ldots, `maxit =
#'     nsteps`. Default value is `20`.
#' @param ... currently not used.
#'
#' @return
#'
#' An object of class `inf_check` that has a [plot()] method.
#'
#' @details
#'
#' [check_infinite_estimates()] attempts to identify the occurrence
#' of infinite estimates in GLMs with binomial responses by
#' successively refitting the model. At each iteration the maximum
#' number of allowed IWLS iterations is fixed starting from 1 to
#' `nsteps` (by setting `control = glm.control(maxit = j)`,
#' where `j` takes values `1, ..., nsteps` in
#' [glm()]. For each value of `maxit`, the estimated
#' asymptotic standard errors are divided to the corresponding ones
#' from `control = glm.control(maxit = 1)`. Then, based on the
#' results in Lesaffre & Albert (1989), if the sequence of ratios in
#' any column of the resultant matrix diverges, then complete or
#' quasi-complete separation occurs and the maximum likelihood
#' estimate for the corresponding parameter has value minus or plus
#' infinity.
#'
#' [check_infinite_estimates()] can also be used to identify the
#' occurrence of infinite estimates in baseline category logit models
#' for nominal responses (see [brglm2::brmultinom()]), and adjacent category
#' logit models for ordinal responses (see [brglm2::bracl()]).
#'
#' @return
#'
#' A matrix inheriting from class `inf_check`, with `nsteps`
#' rows and `p` columns, where `p` is the number of model
#' parameters. A [plot()] method is provided for `inf_check`
#' objects for the easy inspection of the ratios of the standard
#' errors.
#'
#' @note
#'
#' For the definition of complete and quasi-complete separation, see
#' Albert and Anderson (1984). Kosmidis and Firth (2021) prove that
#' the reduced-bias estimator that results by the penalization of the
#' logistic regression log-likelihood by Jeffreys prior takes always
#' finite values, even when some of the maximum likelihood estimates
#' are infinite. The reduced-bias estimates can be computed using the
#' \pkg{brglm2} R package.
#'
#' @seealso [nnet::multinom()],
#'     [detect_separation()],
#'     [brglm2::brmultinom()],
#'     [brglm2::bracl()]
#'
#' @references
#'
#' Lesaffre, E., & Albert, A. (1989). Partial Separation in Logistic
#' Discrimination. *Journal of the Royal Statistical Society. Series B
#' (Methodological)*, **51**, 109-116
#'
#' Kosmidis I. and Firth D. (2021). Jeffreys-prior penalty, finiteness
#' and shrinkage in binomial-response generalized linear
#' models. *Biometrika*, **108**, 71–82
#'
#' @examples
#'
#' # endometrial data from Heinze \& Schemper (2002) (see ?endometrial)
#' data("endometrial", package = "detectseparation")
#' endometrial_ml <- glm(HG ~ NV + PI + EH, data = endometrial,
#'                       family = binomial("probit"))
#' # clearly the maximum likelihood estimate for the coefficient of
#' # NV is infinite
#' (estimates <- check_infinite_estimates(endometrial_ml))
#' plot(estimates)
#'
#'
#' \donttest{
#' # Aligator data (Agresti, 2002, Table~7.1)
#' if (requireNamespace("brglm2", quietly = TRUE)) {
#'     data("alligators", package = "brglm2")
#'     all_ml <- brglm2::brmultinom(foodchoice ~ size + lake , weights = round(freq/3),
#'                          data = alligators, type = "ML", ref = 1)
#'     # Clearly some estimated standard errors diverge as the number of
#'     # Fisher scoring iterations increases
#'     plot(check_infinite_estimates(all_ml))
#'     # Bias reduction the brglm2 R packages can be used to get finite estimates
#'     all_br <- brglm2::brmultinom(foodchoice ~ size + lake , weights = round(freq/3),
#'                          data = alligators, ref = 1)
#'     plot(check_infinite_estimates(all_br))
#' }
#' }
#' @export
check_infinite_estimates.glm <- function(object, nsteps = 20, ...) {
    valid_classes <- c("glm", "brglmFit", "brmultinom")
    is_brmultinom <- inherits(object, "brmultinom")
    if (!inherits(object, valid_classes)) {
        warning("`check_infinite_estimates()` has been designed for objects of class 'glm', 'brglmFit', 'brmultinom'")
    }
    if ((object$family$family != "binomial") & (!is_brmultinom)) {
        warning("`check_infinite_estimates()` has been designed for binomial- or multinomial-response models")
    }
    if (is_brmultinom) {
        betas <- coef(object)
        dims <- dim(betas)
        betasNames <- paste(rep(colnames(betas), dims[1]), rownames(betas), sep = ":")
        betas <- c(betas)
        names(betas) <- betasNames
    }
    else {
        betas <- coef(object)
        betasNames <- names(betas)
    }
    eps <- .Machine$double.eps
    noNA <- !is.na(betas)
    stdErrors <- matrix(0, nsteps, length(betas))
    start <- NULL
    for (i in 1:nsteps) {
        if (is_brmultinom) {
            suppressWarnings(temp.object <- update(object, control = list(maxit = i, epsilon = eps, type = object$type), start = start))
            stdErrors[i, noNA] <- sqrt(diag(vcov(temp.object))[noNA])
        }
        else {
            suppressWarnings(temp.object <- update(object, control = list(maxit = i, epsilon = eps), start = start))
            stdErrors[i, noNA] <- summary(temp.object)$coef[betasNames[noNA], "Std. Error"]
        }
        start <- c(coef(temp.object))
    }
    res <- sweep(stdErrors, 2, stdErrors[1, ], "/")
    colnames(res) <- betasNames
    class(res) <- "inf_check"
    res
}

#' @export
plot.inf_check <- function(x, tol = 1e+2, ...) {
    # heuristic for determining ploting ranges
    sds <- apply(x, 2, sd)
    matplot(x, type = "l", lty = 1, ylim = range(x[, sds < tol]) * c(1, 1.5),
            ylab = "estimate", xlab = "number of iterations")
}

# Copyright (C) 2017- Ioannis Kosmidis, Dirk Schumacher

#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 or 3 of the License
#  (at your option).
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  A copy of the GNU General Public License is available at
#  http://www.r-project.org/Licenses/

#' @title Detect Separation
#'
#' @description
#'
#' Method for [glm()] that tests for data separation and
#' finds which parameters have infinite maximum likelihood estimates
#' in generalized linear models with binomial responses
#'
#' [detect_separation()] is a method for [glm()]
#' that tests for the occurrence of complete or quasi-complete
#' separation in datasets for binomial response generalized linear
#' models, and finds which of the parameters will have infinite
#' maximum likelihood estimates. [detect_separation()]
#' relies on the linear programming methods developed in Konis (2007).
#'
#' @inheritParams stats::glm.fit
#'
#' @aliases detectSeparation print.detect_separation
#'
#' @param x `x` is a design matrix of dimension `n * p`.
#' @param y `y` is a vector of observations of length `n`.
#' @param control a list of parameters controlling separation
#'     detection. See `detect_separation_control()` for
#'     details.
#' @param start currently not used.
#' @param mustart currently not used.
#' @param etastart currently not used.
#' @param singular.ok logical. If `FALSE`, a singular model is an
#'     error.
#'
#' @details
#'
#' Following the definitions in Albert and Anderson (1984), the data
#' for a binomial-response generalized linear model with logistic link
#' exhibit quasi-complete separation if there exists a non-zero
#' parameter vector \eqn{\beta} such that \eqn{X^0 \beta \le 0} and
#' \eqn{X^1 \beta \ge 0}, where \eqn{X^0} and \eqn{X^1} are the
#' matrices formed by the rows of the model matrix $X$ corresponding
#' to zero and non-zero responses, respectively. The data exhibits
#' complete separation if there exists a parameter vector \eqn{\beta} such
#' that the aforementioned conditions are satisfied with strict
#' inequalities. If there are no vectors \eqn{\beta} that can satisfy the
#' conditions, then the data points are said to overlap.
#'
#' If the inverse link function \eqn{G(t)} of a generalized linear
#' model with binomial responses is such that \eqn{\log G(t)} and
#' \eqn{\log (1 - G(t))} are concave and the model has an intercept
#' parameter, then overlap is a necessary and sufficient condition for
#' the maximum likelihood estimates to be finite (see Silvapulle, 1981
#' for a proof). Such link functions are, for example, the logit,
#' probit and complementary log-log.
#'
#' [detect_separation()] determines whether or not the
#' data exhibits (quasi-)complete separation. Then, if separation is
#' detected and the link function \eqn{G(t)} is such that \eqn{\log
#' G(t)} and \eqn{\log (1 - G(t))} are concave, the maximum likelihood
#' estimates has infinite components.
#'
#' [detect_separation()] is a wrapper to the
#' [detect_infinite_estimates()] method. Separation
#' detection, as separation is defined above, takes place using the
#' linear programming methods in Konis (2007) regardless of the link
#' function. The output of those methods is also used to determine
#' which estimates are infinite, unless the link is `"log"`. In the
#' latter case the linear programming methods in Schwendinger et
#' al. (2021) are called to establish if and which estimates are
#' infinite. If the link function is not one of `"logit"`, `"log"`,
#' `"probit"`, `"cauchit"`, `"cloglog"` then a warning is issued.
#'
#' If `separation_type = TRUE` in
#' `detect_separation_control()`, then, whenever
#' separation is detected, [detect_separation()] attempts
#' to distinguish between complete and quasi-complete separation by
#' solving an additional linear program that maximizes the minimum
#' transformed margin. If \eqn{x_i} is the \eqn{i}th row of the model
#' matrix and \eqn{\tilde{y}_i = -1 + 2 y_i}, where \eqn{y_i} is the
#' \eqn{i}th Bernoulli response (the representation to which any
#' binomial data supplied by the user are transformed internally),
#' then \eqn{\bar{x}_i = \tilde{y}_i x_i}. The additional linear
#' program that is solved maximizes \eqn{t} subject to
#' \eqn{\bar{X}\beta \ge t {\bf 1}} and \eqn{-1 \le \beta_j \le
#' 1}. This is equivalent to solving \eqn{\max_\beta \min_i
#' (\bar{x}_i^\top \beta)} subject to \eqn{-1 \le \beta_j \le 1} for
#' all \eqn{j}. A positive optimal value for \eqn{t} implies complete
#' separation, while an optimal value of zero implies
#' quasi-complete separation provided that separation has already been
#' detected. See Konis (2007, Section 1.3 and Chapter 4) for the
#' definitions of separation and the transformed linear programming
#' formulation.
#'
#' The [coefficients()] method extracts a vector of values
#' for each of the model parameters under the following convention:
#' `0` if the maximum likelihood estimate of the parameter is
#' finite, and `Inf` or `-Inf` if the maximum likelihood
#' estimate of the parameter is plus or minus infinity. This
#' convention makes it easy to adjust the maximum likelihood estimates
#' to their actual values by element-wise addition.
#'
#' [detect_separation()] can be passed directly as
#' a method to the [glm()] function. See, examples.
#'
#' [detectSeparation()] is an alias for [detect_separation()].
#'
#' @note
#'
#' For the definition of complete and quasi-complete separation, see
#' Albert and Anderson (1984). Kosmidis and Firth (2021) prove that
#' the reduced-bias estimator that results by the penalization of the
#' logistic regression log-likelihood by Jeffreys prior takes always
#' finite values, even when some of the maximum likelihood estimates
#' are infinite. The reduced-bias estimates can be computed using the
#' \pkg{brglm2} R package.
#'
#' [detect_separation()] was designed in 2017 by Ioannis
#' Kosmidis for the \pkg{brglm2} R package, after correspondence with
#' Kjell Konis, and a port of the `separator()` function had been
#' included in \pkg{brglm2} under the permission of Kjell Konis. In
#' 2020, [detect_separation()] and
#' [check_infinite_estimates()] were moved outside
#' **brglm2** into the dedicated **detectseparation** package. Dirk
#' Schumacher authored the `separator_ROI()` function, which
#' depends on the \pkg{ROI} R package and is now the default
#' implementation used for detecting separation. In 2022, Florian
#' Schwendinger authored the `dielb_ROI()` function for detecting
#' infinite estimates in log-binomial regression, and, with Ioannis
#' Kosmidis, they refactored the codebase to properly accommodate for
#' the support of log-binomial regression.
#'
#' @return
#'
#' A list that inherits from class [`detect_separation`],
#' [`glm`] and [`lm`]. A `print` method is provided for
#' [`detect_separation`] objects. If
#' `detect_separation_control(separation_type = TRUE)` is used and
#' separation is detected, then the returned object has a `complete`
#' component, which is `TRUE` for complete separation and `FALSE` for
#' quasi-complete separation. Otherwise, the `complete` component is
#' `NULL`.
#'
#'
#' @author Ioannis Kosmidis \[aut, cre\] \email{ioannis.kosmidis@warwick.ac.uk}, Dirk Schumacher \[aut\] \email{mail@dirk-schumacher.net}, Florian Schwendinger \[aut\] \email{FlorianSchwendinger@gmx.at}, Kjell Konis \[ctb\] \email{kjell.konis@me.com}
#'
#' @seealso [glm()], [detect_infinite_estimates()], [check_infinite_estimates()], [brglm2::brglm_fit()]
#'
#' @references
#'
#' Konis K. (2007). *Linear Programming Algorithms for Detecting
#' Separated Data in Binary Logistic Regression
#' Models*. DPhil. University of Oxford.
#' <https://ora.ox.ac.uk/objects/uuid:8f9ee0d0-d78e-4101-9ab4-f9cbceed2a2a>
#'
#' Konis K. (2013). safeBinaryRegression: Safe Binary Regression. R
#' package version 0.1-3.
#' <https://CRAN.R-project.org/package=safeBinaryRegression>
#'
#' Kosmidis I. and Firth D. (2021). Jeffreys-prior penalty, finiteness
#' and shrinkage in binomial-response generalized linear
#' models. *Biometrika*, **108**, 71–82. \doi{10.1093/biomet/asaa052}
#'
#' Silvapulle, M. J. (1981).  On the Existence of Maximum Likelihood
#' Estimators for the Binomial Response Models.  *Journal of the Royal
#' Statistical Society. Series B (Methodological)*, **43**, 310–313.
#' <https://www.jstor.org/stable/2984941>
#'
#' Schwendinger, F., Grün, B. & Hornik, K. (2021). A comparison of
#' optimization solvers for log binomial regression including conic
#' programming.  *Computational Statistics*, **36**,
#' 1721–1754. \doi{10.1007/s00180-021-01084-5}
#'
#'
#' @examples
#'
#' # endometrial data from Heinze \& Schemper (2002) (see ?endometrial)
#' data("endometrial", package = "detectseparation")
#' endometrial_sep <- glm(HG ~ NV + PI + EH, data = endometrial,
#'                        family = binomial("logit"),
#'                        method = "detect_separation")
#' endometrial_sep
#' # The maximum likelihood estimate for NV is infinite
#' summary(update(endometrial_sep, method = "glm.fit"))
#'
#' # If we want to further check for the type of separation (complete
#' # or quasi-complete) we can do
#' endometrial_sep_type <- update(endometrial_sep, separation_type = TRUE)
#' endometrial_sep
#' endometrial_sep_type
#'
#' \donttest{
#' # Example inspired by unpublished microeconometrics lecture notes by
#' # Achim Zeileis https://eeecon.uibk.ac.at/~zeileis/
#' # The maximum likelihood estimate of sourhernyes is infinite
#' if (requireNamespace("AER", quietly = TRUE)) {
#'     data("MurderRates", package = "AER")
#'     murder_sep <- glm(I(executions > 0) ~ time + income +
#'                       noncauc + lfp + southern, data = MurderRates,
#'                       family = binomial(), method = "detect_separation")
#'     murder_sep
#'     # which is also evident by the large estimated standard error for NV
#'     murder_glm <- update(murder_sep, method = "glm.fit")
#'     summary(murder_glm)
#'     # and is also revealed by the divergence of the NV column of the
#'     # result from the more computationally intensive check
#'     plot(check_infinite_estimates(murder_glm))
#'     # Mean bias reduction via adjusted scores results in finite estimates
#'     if (requireNamespace("brglm2", quietly = TRUE))
#'         update(murder_glm, method = brglm2::brglm_fit)
#' }
#' }
#' @export
detect_separation <- function(x, y, weights = NULL,
                              start = NULL, etastart = NULL,  mustart = NULL,
                              offset = NULL, family = gaussian(),
                              control = list(), intercept = TRUE, singular.ok = TRUE) {
    log_link <- isTRUE(family$link == "log")
    if (isTRUE(family$family == "binomial")) {
        reliable_links <- c("logit", "log", "probit", "cauchit", "cloglog")
        if (!isTRUE(family$link %in% reliable_links)) {
            warning("`detect_separation()` results may be unreliable for binomial-response GLMs",
                    " with links other than ", paste(shQuote(reliable_links), collapse = ", "))
        }
        if (log_link) {
            message("Data separation in log-binomial models does not necessarily result in infinite estimates")
        }
    }
    else {
        warning("`detect_separation()` has been developed for use with binomial-response GLMs")
    }
    out <- .detect_infinite_estimates(x = x, y = y, weights = weights, start = start,
                                      etastart = etastart,  mustart = mustart,
                                      offset = offset, family = family, control = control,
                                      intercept = control, singular.ok = singular.ok,
                                      log_link = FALSE)
    out$complete <- NULL
    if (isTRUE(out$control$separation_type) && isTRUE(out$outcome)) {
        complete <- separation_type_ROI(x = out$x, y = out$y,
                                        solver = out$control$solver,
                                        tolerance = out$control$tolerance,
                                        solver_control = out$control$solver_control)
        out$complete <- complete$outcome
        if (is.na(complete$outcome)) {
            warning("unexpected result from `separation_type_ROI` with solver: ", out$control$solver, "\n")
        }
    }
    if (log_link) {
        # test for existence using the linear program in Schwendinger et al (2021)
        out$coefficients <- .detect_infinite_estimates(x = x, y = y, weights = weights, start = start,
                                                       etastart = etastart,  mustart = mustart,
                                                       offset = offset, family = family, control = control,
                                                       intercept = control, singular.ok = singular.ok,
                                                       log_link = TRUE)$coefficients
    }
    class(out) <- out$class <- "detect_separation"
    out
}

#' @method print detect_separation
#' @export
print.detect_separation <- function(x, digits = max(5L, getOption("digits") - 3L), ...) {
    cat("Implementation:", x$control$implementation, "| ")
    if (identical(x$control$implementation, "ROI")) {
        cat("Solver:", x$control$solver, "\n")
    }
    else {
        cat("Linear program:", x$control$linear_program, "| Purpose:", x$control$purpose, "\n")
    }
    if (is.null(x$complete)) {
        str <- "\n"
    } else {
        if (is.na(x$complete)) {
            str <- "(type undetermined)\n"
        }
        else {
            str <- ifelse(isTRUE(x$complete), "(complete)\n", "(quasi-complete)\n")
        }
    }
    cat("Separation:", x$outcome, str)
    if (!is.null(x$coefficients)) {
        cat("Existence of maximum likelihood estimates\n")
        print(coefficients(x))
        cat("0: finite value, Inf: infinity, -Inf: -infinity\n")
    }
}

#' Auxiliary function for the [glm()] interface when
#' `method` is [detect_separation()]
#'
#' Typically only used internally by [detect_separation()]
#' but may be used to construct a `control` argument.
#'
#' @aliases detectSeparationControl
#' @param implementation should the implementation using `ROI` or the
#'     implementation using `lpSolveAPI` be used? Default is `ROI`.
#' @param tolerance maximum absolute variable value from the linear
#'     program, before separation is declared. Default is `1e-04`.
#' @param linear_program should [detect_separation()] solve the
#'     `"primal"` (default) or `"dual"` linear program for separation
#'     detection? Only relevant if `implementation = "lpSolveAPI"`.
#' @param purpose should [detect_separation()] simply `"test"` for
#'     separation or also `"find"` (default) which parameters are
#'     infinite? Only relevant if `implementation = "lpSolveAPI"`.
#' @param solver should the linear program be solved using the
#'     `"lpsolve"` (using the `ROI.plugin.lpsolve` package; default)
#'     or another solver? Alternative solvers are `"glpk"`, `"cbc"`,
#'     `"clp"`, `"cplex"`, `"ecos"`, `"gurobi"`, `"scs"`,
#'     `"symphony"`. If the corresponding package
#'     `ROI.plugin.[solver]` is not installed then the user will be
#'     prompted to install it before continuing.
#' @param solver_control a list with additional control parameters for
#'     the `"solver"`. This is solver specific, so consult the
#'     corresponding documentation. Default is `list()` unless
#'     `solver` is `"alabama"` when the default is `list(start =
#'     rep(0, p))`, where `p` is the number of parameters.
#' @param separation_type logical. Should [detect_separation()]
#'     attempt to distinguish complete from quasi-complete separation
#'     after separation has been detected? If `TRUE` and separation is
#'     detected, then an additional linear program is solved. Default
#'     is `FALSE`.
#'
#'
#' @return
#'
#' A list with the supplied `linear_program`, `solver`,
#' `solver_control`, `purpose`, `tolerance`, `separation_type`,
#' and `implementation`. The returned list is intended to be passed to
#' the `control` argument of [detect_separation()] and
#' [detect_infinite_estimates()].
#'
#' @references
#'
#' Konis K. (2007). *Linear Programming Algorithms for Detecting
#' Separated Data in Binary Logistic Regression Models*. DPhil.
#' University of Oxford.
#' <https://ora.ox.ac.uk/objects/uuid:8f9ee0d0-d78e-4101-9ab4-f9cbceed2a2a>
#'
#' @examples
#' data("endometrial", package = "detectseparation")
#' ctrl <- detect_separation_control(separation_type = TRUE)
#' glm(HG ~ NV + PI + EH, data = endometrial,
#'     family = binomial("logit"),
#'     method = "detect_separation",
#'     control = ctrl)
#'
#' @export
detect_separation_control <- function(implementation = c("ROI", "lpSolveAPI"),
                                      solver = "lpsolve",
                                      linear_program = c("primal", "dual"),
                                      purpose = c("find", "test"),
                                      tolerance = 1e-04,
                                      solver_control = list(),
                                      separation_type = FALSE) {
    implementation <- match.arg(implementation)
    purpose <- match.arg(purpose)
    linear_program <- match.arg(linear_program)
    check_ROI_solver(solver)
    list(linear_program = linear_program,
         solver = solver,
         solver_control = solver_control,
         purpose = purpose,
         tolerance = tolerance,
         separation_type = separation_type,
         implementation = implementation)
}

## endometrial data from Heinze \& Schemper (2002) (see ?endometrial)
data("endometrial", package = "detectseparation")
endometrial_separation <- glm(HG ~ I(-NV) + PI + EH, data = endometrial,
                              family = binomial("cloglog"),
                              method = "detect_separation")

## The lizards example from ?brglm::brglm
data("lizards", package = "detectseparation")
lizards_separation <- glm(cbind(grahami, opalinus) ~ height + diameter +
                              light + time, family = binomial(logit), data = lizards,
                          method = "detect_separation")

## Check that the coefficients are as expected
expect_equal(coef(endometrial_separation), c(0L, -Inf, 0L, 0L), check.attributes = FALSE)
expect_equal(coef(lizards_separation), rep(0L, 6), check.attributes = FALSE)

## endometrial using lpSolveAPI
endometrial_separation_lpsolve <- glm(HG ~ I(-NV) + PI + EH, data = endometrial,
                                      family = binomial("cloglog"),
                                      method = "detect_separation",
                                      implementation = "lpSolveAPI")

## testing lpSolveAPI implementation options
endometrial_separation_lpsolve2 <- update(endometrial_separation_lpsolve,
                                          linear_program = "dual",
                                          purpose = "test")

## test output
expect_stdout(print(endometrial_separation), "ROI \\| Solver: lpsolve")
expect_stdout(print(endometrial_separation), "0: finite value, Inf: infinity, -Inf: -infinity")
expect_stdout(print(endometrial_separation), "Separation: TRUE")
expect_stdout(print(lizards_separation), "ROI \\| Solver: lpsolve")
expect_stdout(print(lizards_separation), "0: finite value, Inf: infinity, -Inf: -infinity")
expect_stdout(print(endometrial_separation_lpsolve), "Implementation: lpSolveAPI \\| Linear program: primal \\| Purpose: find")
expect_stdout(print(endometrial_separation_lpsolve), "Separation: TRUE")
expect_stdout(print(endometrial_separation_lpsolve2), "Implementation: lpSolveAPI \\| Linear program: dual \\| Purpose: test \\nSeparation: TRUE")

## Test separation type detection
counterexample_x <- rbind(c(1, -1, 2),
                          c(1, -1, -2),
                          c(1, -2, -2),
                          c(1, 2, 2))
counterexample_y <- c(0, 1, 1, 1)

counterexample_separation <- detect_separation(x = counterexample_x,
                                               y = counterexample_y,
                                               family = binomial(),
                                               control = list(separation_type = TRUE))
expect_true(counterexample_separation$outcome)
expect_true(counterexample_separation$complete)
expect_stdout(print(counterexample_separation), "Separation: TRUE \\(complete\\)")

counterexample_separation_lpsolve <- detect_separation(x = counterexample_x,
                                                       y = counterexample_y,
                                                       family = binomial(),
                                                       control = list(implementation = "lpSolveAPI",
                                                                      linear_program = "primal",
                                                                      purpose = "test",
                                                                      separation_type = TRUE))
expect_true(counterexample_separation_lpsolve$outcome)
expect_true(counterexample_separation_lpsolve$complete)

overlap_x <- cbind(1, c(-1, 0, 1, 2))
overlap_y <- c(0, 1, 0, 1)
overlap_separation <- detect_separation(x = overlap_x,
                                        y = overlap_y,
                                        family = binomial(),
                                        control = list(separation_type = TRUE))
expect_false(overlap_separation$outcome)
expect_null(overlap_separation$complete)

## Test aliasing
endometrial_separation_a <- glm(HG ~ NV + I(2 * NV) + PI + EH, data = endometrial,
                                family = binomial("cloglog"),
                                method = "detect_separation")
expect_equal(coef(endometrial_separation_a), c(0L, Inf, NA, 0L, 0L), check.attributes = FALSE)


## Test EMPTY fits
endometrial_separation_n <- glm(HG ~ 0, data = endometrial,
                                family = binomial("cloglog"),
                                method = "detect_separation")
expect_null(coef(endometrial_separation_n))
expect_false(endometrial_separation_n$outcome)


## Test log-binomial models
data("silvapulle1981", package = "detectseparation")

expect_message(m1sep <- glm(y ~ ghqs, data = silvapulle1981, family = binomial("log"),
                            method = "detect_separation"), pattern = "necessarily result in")

m1inf <- glm(y ~ ghqs, data = silvapulle1981, family = binomial("log"),
             method = "detect_infinite_estimates")

expect_message(m1sep_type <- glm(y ~ ghqs, data = silvapulle1981, family = binomial("log"),
                                 method = "detect_separation", separation_type = TRUE),
               pattern = "necessarily result in")

expect_equal(unname(coef(m1sep)), unname(coef(m1inf)))
expect_equal(unname(coef(m1sep)), c(0, 0))
expect_true(m1sep_type$outcome)
expect_false(m1sep_type$complete)





## ## hepatitis
## data("hepatitis", package = "pmlr")
## hepat <- hepatitis
## hepat$type <- with(hepat, factor(1 - HCV * nonABC + HCV + 2 * nonABC))
## hepat$type <- factor(hepat$type, labels = c("noDisease", "C", "nonABC"))
## y <- rowSums(hepat$counts*nnet::class.ind(hepat$type)[,c(1, 3)])
## glm(y/counts ~ group * time, data = hepat, weights = counts, family = binomial(),
##     method = "detect_separation")
## dd <- data.frame(y = c(1,1,1,0,0), x = c(1,2,4,4,5), off = c(1,2, 2,4,3))
## summary(glm(y ~ x + offset(off), family = binomial(), data = dd))

## test defaults
out0 <- detect_separation_control()

expect_equal(out0$linear_program, "primal")
expect_equal(out0$solver, "lpsolve")
expect_equal(out0$purpose, "find")
expect_equal(out0$tolerance, 1e-04)
expect_equal(out0$implementation, "ROI")

## test arguments
out1 <- detect_separation_control(implementation = "lpSolveAPI", solver = "alabama", linear_program = "dual", solver_control = list(abs = 2), tolerance = 1e-07)

expect_equal(out1$linear_program, "dual")
expect_equal(out1$solver, "alabama")
expect_equal(out1$purpose, "find")
expect_equal(out1$tolerance, 1e-07)
expect_equal(out1$implementation, "lpSolveAPI")

# Some convenience names

#' @rdname detect_separation
#' @export
detectSeparation <- detect_separation

#' @rdname detect_separation_control
#' @export
detectSeparationControl <- detect_separation_control

#' @rdname check_infinite_estimates
#' @export
checkInfiniteEstimates <- check_infinite_estimates

#' @rdname detect_infinite_estimates
#' @export
detectInfiniteEstimates <- detect_infinite_estimates

#' Habitat preferences of lizards
#'
#' @format A data frame with 23 rows and 6 columns:
#'
#' * `grahami`. count of grahami lizards
#' * `opalinus`. count of opalinus lizards
#' * `height`. a factor with levels `<5ft`, `>=5ft`
#' * `diameter`. a factor with levels `<=2in`, `>2in`
#' * `light`. a factor with levels `sunny`, `shady`
#' * `time`. a factor with levels `early`, `midday`, `late`
#'
#' The variables `grahami` and `opalinus` are counts of two lizard
#' species at two different perch heights, two different perch
#' diameters, in sun and in shade, at three times of day.
#'
#' @seealso
#'
#' [detect_separation()]
#'
#' @source
#'
#' McCullagh P, Nelder J A (1989) _Generalized Linear
#' Models_ (2nd Edition).  London: Chapman and Hall.
#'
#' Originally from
#'
#' Schoener T W (1970) Nonsynchronous spatial overlap of lizards
#' in patchy habitats.  _Ecology_ *51*, 408-418.
#'
"lizards"

#' Histology grade and risk factors for 79 cases of endometrial cancer
#'
#' @format
#'
#' A data frame with 79 rows and 4 variables:
#'
#' * `NV`: neovasculization with coding 0 for absent and 1 for present
#' * `PI`: pulsality index of arteria uterina
#' * `EH`: endometrium height
#' * `HG`: histology grade with coding 0 for low grade and 1 for high grade
#'
#' @source
#'
#' The packaged data set was downloaded in `.dat` format from
#' <https://users.stat.ufl.edu/~aa/glm/data/>. The latter link
#' provides the data sets used in Agresti (2015).
#'
#' The endometrial data set was first analyzed in Heinze and
#' Schemper (2002), and was originally provided by Dr
#' E. Asseryanis from the Medical University of Vienna.
#'
#' @seealso
#'
#' [detect_separation()]
#'
#'
#' @references
#'
#' Agresti A (2015). *Foundations of Linear and Generalized Linear
#' Models*.  Wiley Series in Probability and Statistics. Wiley.
#'
#' Heinze G, Schemper M (2002). A Solution to the Problem of
#' Separation in Logistic Regression. *Statistics in Medicine*,
#' **21**, 2409–2419. \doi{10.1002/sim.1047}.
#'
#' Kosmidis I, Firth D (2021). Jeffreys-prior penalty, finiteness
#' and shrinkage in binomial-response generalized linear
#' models. *Biometrika*, **108**, 71-82. \doi{10.1093/biomet/asaa052}.
#'
#'
"endometrial"

#' Separation Example Presented in Silvapulle (1981)
#'
#' @description
#' Separation example presented in Silvapulle (1981).
#'
#' @format
#'
#' A data frame with 35 rows and 2 variables:
#'
#' * `y`: a factor with the levels `case` and `none-case`, giving the
#'        outcome of a standardized psychiatric interview
#' * `ghqs`: an integer giving the general health questionnaire score.
#'
#' @seealso
#'
#' [detect_infinite_estimates()]
#'
#' @references
#'
#' Silvapulle, M. J. (1981).
#' On the Existence of Maximum Likelihood Estimators for the Binomial Response Models.
#' Journal of the Royal Statistical Society. Series B (Methodological), 43(3), 310–313.
#' <https://www.jstor.org/stable/2984941>
#'
"silvapulle1981"

#  separation_type_ROI:
#  Copyright (C) 2026- Ioannis Kosmidis
#
#  separator_ROI:
#  Copyright (C) 2021- Dirk Schumacher, Ioannis Kosmidis
#
#  separator_lpSolveAPI: Port of the separator function from the safeBinaryRegression (version 0.1-3) R package (see safeBinaryRegression/R/separator.R)
#  Copyright (C) 2009-2013 Kjell Konis; Copyright (C) 2017- Ioannis Kosmidis
#
#  dielb_ROI:
#  Copyright (C) 2022- Florian Schwendinger, Ioannis Kosmidis
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 or 3 of the License
#  (at your option).
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  A copy of the GNU General Public License is available at
#  http://www.r-project.org/Licenses/

#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 or 3 of the License
#  (at your option).
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  A copy of the GNU General Public License is available at
#  http://www.r-project.org/Licenses/

separator_ROI <- function(x, y,
                          solver = "lpsolve",
                          tolerance = 1e-03,
                          solver_control = list(),
                          ...) {
    # The model here is based on Konis (2007), chapter 4. In particular
    # sections 4.2, 4.4.3 and 4.4.4
    #
    # build the ROI model
    # transform the model matrix so that all constraints are >=
    # that should also work with doubles
    y[y == 0] <- -1
    X_bar <- x * y
    m <- ncol(X_bar)
    n <- nrow(X_bar)
    constraints <- ROI::L_constraint(X_bar, rep.int(">=", n), rep.int(0, n))
    bounds <- ROI::V_bound(li = seq_len(m),
                           lb = rep.int(-1, m),
                           ui = seq_len(m),
                           ub = rep.int(1, m))
    # max t(rep.int(1, n)) %*% X_bar %*% beta = colSums(X_bar) %*% beta
    # subject to X_bar >= 0
    # beta between -1 and 1
    opt_model <- ROI::OP(objective = colSums(X_bar),
                         constraints = constraints,
                         types = rep.int("C", m),
                         bounds = bounds,
                         maximum = TRUE)

    # if the LP is unbounded, seperation exists. Otherwise an optiomal solution
    # with obj. value 0 exists.
    result <- ROI::ROI_solve(opt_model, solver = solver, control = solver_control)
    # compare to 0 zero with tolerance
    sol <- ROI::solution(result, "primal")
    names(sol) <- colnames(x)
    has_separation <- any(abs(sol) > tolerance, na.rm = TRUE)
    # an optimal solution should always exist
    if (!isTRUE(ROI::solution(result, "status_code") == 0L)) {
        has_separation <- NA
    }
    list(outcome = has_separation,
         beta = sol)
}

# detect infinite estimates log binomial (dielb)
dielb_ROI <- function(x, y,
                      solver = "lpsolve",
                      tolerance = 1e-03,
                      solver_control = list(),
                      ...) {
    # Use linear program from
    #
    # Schwendinger, F., Grün, B. & Hornik, K. A comparison of
    # optimization solvers for log binomial regression including
    # conic programming. Comput Stat 36, 1721–1754
    # (2021). https://doi.org/10.1007/s00180-021-01084-5

    direction <- ifelse(y == 0, ROI::leq(1), ROI::eq(1))
    op <- ROI::OP(objective = -colSums(x), maximum = TRUE,
                  constraints = ROI::L_constraint(x, direction, double(nrow(x))),
                  bounds = ROI::V_bound(ld = -Inf, ud = Inf, nobj = ncol(x)))

    if (isTRUE(solver == "lpsolve")) {
        control <- list(pivoting = "firstindex", simplextype = c("primal", "primal"))
        solver_control <- utils::modifyList(control, solver_control)
    }

    result <- ROI::ROI_solve(op, solver, control = solver_control)
    sol <- ROI::solution(result)
    names(sol) <- colnames(x)
    has_infinite_estimates <-  !isTRUE(ROI::solution(result, "status_code") == 0L)
    list(outcome = has_infinite_estimates,
         beta = sol)
}

#  Port of the separator function from the safeBinaryRegression
#  (version 0.1-3) R package (see safeBinaryRegression/R/separator.R)
separator_lpSolveAPI <- function(x, y,
                                 linear_program = c("primal", "dual"),
                                 purpose = c("test", "find"),
                                 tolerance = 1e-03,
                                 ...) {
    n <- dim(x)[1L]
    p <- dim(x)[2L]
    p_seq <- seq.int(p)
    zeros <- rep.int(0, n)
    dimnames(x) <- NULL
    y.bar <- -sign(y - 0.5)
    x.bar <- y.bar * x
    ans <- list()
    linear_program <- match.arg(linear_program)
    purpose <- match.arg(purpose)
    if (linear_program == "primal" && purpose == "test") {
        lp <- lpSolveAPI::make.lp(n, p)
        for(j in p_seq) {
            status <- lpSolveAPI::set.column(lp, j, x.bar[, j])
        }
        status <- lpSolveAPI::set.rhs(lp,  zeros)
        status <- lpSolveAPI::set.constr.type(lp, rep.int(1L, n))
        status <- lpSolveAPI::set.objfn(lp, -colSums(x.bar))
        status <- lpSolveAPI::set.bounds(lp, lower = rep(-Inf, p), upper = rep(Inf, p))
        control <- lpSolveAPI::lp.control(lp, pivoting = "firstindex", sense = "max",
                                          simplextype = c("primal", "primal"))
        status <- lpSolveAPI::solve.lpExtPtr(lp)
        if (status == 0) {
            ans$outcome <- FALSE
        }
        else {
            if (status == 3) {
                ans$outcome <- TRUE
            }
            else {
                ans$outcome <- NA
            }
        }
    }
    if (linear_program == "primal" && purpose == "find") {
        lp <- lpSolveAPI::make.lp(n, p)
        for (j in p_seq) {
            status <- lpSolveAPI::set.column(lp, j, x.bar[, j])
        }
        status <- lpSolveAPI::set.rhs(lp, zeros)
        status <- lpSolveAPI::set.constr.type(lp, rep.int(1, n))
        status <- lpSolveAPI::set.objfn(lp, -colSums(x.bar))
        status <- lpSolveAPI::set.bounds(lp, lower = rep.int(-1, p), upper = rep.int(1, p))
        control <- lpSolveAPI::lp.control(lp, pivoting = "firstindex", sense = "max",
                              simplextype = c("primal", "primal"))
        status <- lpSolveAPI::solve.lpExtPtr(lp)
        if (status != 0) {
            ans$outcome <- NA
        }
        beta <- lpSolveAPI::get.variables(lp)
        if (any(abs(beta) > tolerance)) {
            ans$outcome <- TRUE
        }
        else {
            ans$outcome <- FALSE
        }
        ans$beta <- beta
    }
    if (linear_program == "dual" && purpose == "test") {
        lp <- lpSolveAPI::make.lp(p, n)
        for (j in 1:n) {
            status <- lpSolveAPI::set.column(lp, j, x.bar[j, ])
        }
        status <- lpSolveAPI::set.rhs(lp, -colSums(x.bar))
        status <- lpSolveAPI::set.constr.type(lp, rep.int(3, p))
        status <- lpSolveAPI::set.objfn(lp, zeros)
        status <- lpSolveAPI::set.bounds(lp, lower = zeros, upper = rep(Inf, n))
        control <- lpSolveAPI::lp.control(lp, pivoting = "firstindex", sense = "min",
                                          simplextype = c("primal", "primal"))
        status <- lpSolveAPI::solve.lpExtPtr(lp)
        if (status == 0) {
            ans$outcome <- FALSE
        }
        else {
            if (status == 2) {
                ans$outcome <- TRUE
            }
            else {
                ans$outcome <- NA
            }
        }
    }
    if (linear_program == "dual" && purpose == "find") {
        lp <- lpSolveAPI::make.lp(p, n + 2*p)
        for (j in 1:n) {
            status <- lpSolveAPI::set.column(lp, j, x.bar[j, ])
        }
        for (j in p_seq) {
            status <- lpSolveAPI::set.column(lp, n+j, -1.0, j)
        }
        # IK, 12 April 2017: p_seq instead 1:n below;
        # safeBinaryRegression:::separator (version 0.1-3) has 1:n
        for (j in p_seq) {
            status <- lpSolveAPI::set.column(lp, n+p+j, 1.0, j)
        }
        b <- -colSums(x.bar)
        status <- lpSolveAPI::set.rhs(lp, b)
        status <- lpSolveAPI::set.constr.type(lp, rep.int(3, p))
        status <- lpSolveAPI::set.objfn(lp, rep.int(c(0.0, 1.0), c(n, 2*p)))
        status <- lpSolveAPI::set.bounds(lp, lower = rep.int(0, n + 2*p), upper = rep(Inf, n + 2*p))
        control <- lpSolveAPI::lp.control(lp, pivoting = "firstindex", sense = "min",
                                          simplextype = c("primal", "primal"))
        basis <- p_seq
        basis[b >= 0.0] <- basis[b >= 0.0] + p
        status <- lpSolveAPI::set.basis(lp, -(n + p + basis))
        status <- lpSolveAPI::solve.lpExtPtr(lp)
        beta <- lpSolveAPI::get.dual.solution(lp)[2:(p+1)]
        if (any(abs(beta) > tolerance)) {
            ans$outcome <- TRUE
        }
        else {
            ans$outcome <- FALSE
        }
        ans$beta <- beta
    }
    ans
}

separation_type_ROI <- function(x, y,
                         solver = "lpsolve",
                         tolerance = 1e-03,
                         solver_control = list(),
                         ...) {
    # The model here is based on the transformed design matrix X_bar and
    # solves a max-min-margin problem. In particular, it maximizes the
    # minimum margin t subject to X_bar %*% beta >= t. A strictly positive
    # optimal value for t implies complete separation.
    #
    # build the ROI model
    # transform the model matrix so that all constraints are >=
    # that should also work with doubles
    y[y == 0] <- -1
    X_bar <- x * y
    m <- ncol(X_bar)
    n <- nrow(X_bar)
    constraints <- ROI::L_constraint(cbind(X_bar, -1), rep.int(">=", n), rep.int(0, n))
    bounds <- ROI::V_bound(li = seq_len(m),
                           lb = rep.int(-1, m),
                           ui = seq_len(m),
                           ub = rep.int(1, m),
                           nobj = m + 1)
    # max t
    # subject to X_bar %*% beta >= t
    # beta between -1 and 1
    opt_model <- ROI::OP(objective = c(rep.int(0, m), 1),
                         constraints = constraints,
                         types = rep.int("C", m + 1),
                         bounds = bounds,
                         maximum = TRUE)
    result <- ROI::ROI_solve(opt_model, solver = solver, control = solver_control)
    status <- ROI::solution(result, "status_code")
    sol <- ROI::solution(result, "primal")
    if (length(sol) != m + 1L) {
        sol <- rep(NA_real_, m + 1L)
    }
    beta <- sol[seq_len(m)]
    names(beta) <- colnames(x)
    minimum_margin <- unname(sol[m + 1])
    has_complete_separation <- isTRUE(minimum_margin > tolerance)
    if (!isTRUE(status == 0L)) {
        has_complete_separation <- NA
    }
    list(outcome = has_complete_separation,
         beta = beta,
         minimum_margin = minimum_margin)
}

#' detectseparation: Methods for Detecting and Checking for Separation
#' and Infinite Maximum Likelihood Estimates
#'
#' \pkg{detectseparation} provides pre-fit and post-fit methods for
#' the detection of separation and of infinite maximum likelihood
#' estimates in binomial response generalized linear models.
#'
#' The key methods are [detect_separation()] and
#' [check_infinite_estimates()].
#'
#' @seealso [detect_separation()], [check_infinite_estimates()]
#'
#' @name detectseparation
#' @import ROI.plugin.lpsolve
#' @importFrom stats coef coefficients gaussian update vcov sd binomial nobs
#' @importFrom graphics matplot
#'
"_PACKAGE"

#' Generic method for checking for infinite estimates
#' @param object a fitted model object (e.g. the result of a
#'     [glm()] call).
#' @param ... other options to be passed to the method.
#'
#' @seealso check_infinite_estimates.glm
#' @export
check_infinite_estimates <- function(object, ...) {
    UseMethod("check_infinite_estimates")
}

data("endometrial", package = "detectseparation")

endometrial_ml <- glm(HG ~ NV + PI + EH, data = endometrial, family = binomial("logit"))
inf_check <- check_infinite_estimates(endometrial_ml)

## NV is infinite
expect_true(max(diff(inf_check[, "NV"])) > 1e+06)

## Make sure a plot method is available for inf_check objects
expect_true("plot.inf_check" %in% methods(class = "inf_check"))

data("silvapulle1981", package = "detectseparation")

m_inf_log <- glm(y ~ ghqs, data = silvapulle1981, family = binomial("log"),
                 method = "detect_infinite_estimates")
m_inf_logit <- glm(y ~ ghqs, data = silvapulle1981, family = binomial("logit"),
                   method = "detect_infinite_estimates")

expect_equal(unname(coef(m_inf_log)), c(0, 0))
expect_equal(unname(coef(m_inf_logit)), c(-Inf, Inf))




## Empty model
mempty <- update(m_inf_log, y ~ -1)
expect_null(coef(mempty))
expect_false(mempty$outcome)

if (requireNamespace("AER", quietly = TRUE)) {
    data("MurderRates", package = "AER")
    murder_formula <- I(executions > 0) ~ time + income + noncauc + lfp + southern
    murder_sep_lpsolve <- glm(murder_formula, data = MurderRates,
                              family = binomial(),
                              method = "detect_separation",
                              implementation = "ROI",
                              solver = "lpsolve")

    murder_sep_glpk <- update(murder_sep_lpsolve,
                              implementation = "ROI",
                              solver = "glpk")

    murder_sep_alabama <- update(murder_sep_lpsolve,
                                 implementation = "ROI",
                                 solver = "alabama")

    murder_sep_ecos <- update(murder_sep_lpsolve,
                                 implementation = "ROI",
                                 solver = "ecos")
    
    murder_sep_alabama1 <- update(murder_sep_lpsolve,
                                  implementation = "ROI",
                                  solver = "alabama",
                                  solver_control = list(start = rep(1, 6)))

    murder_sep_lpSolveAPI <- update(murder_sep_lpsolve,
                                    implementation = "lpSolveAPI")


    expect_equal(coef(murder_sep_lpsolve), coef(murder_sep_glpk))
    expect_equal(coef(murder_sep_lpsolve), coef(murder_sep_alabama))
    expect_equal(coef(murder_sep_lpsolve), coef(murder_sep_ecos))
    expect_equal(coef(murder_sep_lpsolve), coef(murder_sep_alabama1))
    expect_equal(coef(murder_sep_lpsolve), coef(murder_sep_lpSolveAPI))

}


data("endometrial", package = "detectseparation")
endo_sep_konis <- glm(HG ~ I(-NV) + PI + EH, data = endometrial,
                      family = binomial("cloglog"),
                      method = "detect_separation",
                      implementation = "lpSolveAPI")

endo_sep_new <- update(endo_sep_konis, implementation = "ROI")


expect_identical(endo_sep_konis$separation, endo_sep_new$separation)
expect_equal(coef(endo_sep_konis), coef(endo_sep_new))    

settings <- expand.grid(lp = c("dual", "primal"),
                        purpose = c("find", "test"),
                        stringsAsFactors = FALSE)


if (requireNamespace("AER", quietly = TRUE)) {

    data("MurderRates", package = "AER")
    murder_formula <- I(executions > 0) ~ time + income + noncauc + lfp + southern
    murder_sep <- as.list(numeric(4))
    for (j in seq.int(nrow(settings))) {
        murder_sep[[j]] <- glm(murder_formula, data = MurderRates,
                               family = binomial(),
                               method = "detect_separation",
                               implementation = "lpSolveAPI",
                               linear_program = settings[j, "lp"],
                               purpose = settings[j, "purpose"])
    }
    names(murder_sep) <- apply(settings, 1, paste, collapse = "-")
    ## test that lpSolveAPI implementation returns the same result across linear_program and purpose settings [MurderRates]"
    expect_equal(coef(murder_sep[["dual-find"]]), coef(murder_sep[["primal-find"]]))
    expect_equal(murder_sep[["dual-find"]]$separation, murder_sep[["primal-find"]]$separation)
    expect_equal(murder_sep[["dual-find"]]$separation, murder_sep[["primal-test"]]$separation)
    expect_equal(murder_sep[["dual-find"]]$separation, murder_sep[["dual-test"]]$separation)
}

data("endometrial", package = "detectseparation")
endo_sep <- as.list(numeric(4))
for (j in seq.int(nrow(settings))) {
    endo_sep[[j]] <- glm(HG ~ I(-NV) + PI + EH,
                         data = endometrial,
                         family = binomial("cloglog"),
                         method = "detect_separation",
                         implementation = "lpSolveAPI",
                         linear_program = settings[j, "lp"],
                         purpose = settings[j, "purpose"])
}
names(endo_sep) <- apply(settings, 1, paste, collapse = "-")
## test that lpSolveAPI implementation returns the same result across linear_program and purpose settings [endometrial]
expect_equal(coef(endo_sep[["dual-find"]]), coef(endo_sep[["primal-find"]]))
expect_equal(endo_sep[["dual-find"]]$separation, endo_sep[["primal-find"]]$separation)
expect_equal(endo_sep[["dual-find"]]$separation, endo_sep[["primal-test"]]$separation)
expect_equal(endo_sep[["dual-find"]]$separation, endo_sep[["dual-test"]]$separation)

data("lizards", package = "detectseparation")
lizo_sep <- as.list(numeric(4))
for (j in seq.int(nrow(settings))) {
    lizo_sep[[j]] <- glm(cbind(grahami, opalinus) ~ height + diameter +
                             light + time,
                         data = lizards,
                         family = binomial("logit"),
                         method = "detect_separation",
                         implementation = "lpSolveAPI",
                         linear_program = settings[j, "lp"],
                         purpose = settings[j, "purpose"])
}
names(lizo_sep) <- apply(settings, 1, paste, collapse = "-")
## test that lpSolveAPI implementation returns the same result across linear_program and purpose settings [lizards]
expect_equal(coef(lizo_sep[["dual-find"]]), coef(lizo_sep[["primal-find"]]))
expect_equal(lizo_sep[["dual-find"]]$separation, lizo_sep[["primal-find"]]$separation)
expect_equal(lizo_sep[["dual-find"]]$separation, lizo_sep[["primal-test"]]$separation)
expect_equal(lizo_sep[["dual-find"]]$separation, lizo_sep[["dual-test"]]$separation)

---
title: "Detecting separation and infinite estimates in log binomial regression"
author: "Florian Schwendinger"
date: "`r Sys.Date()`"
output: 
  rmarkdown::html_vignette:
    number_sections: true
bibliography: detectseparation.bib
link-citations: yes
vignette: >
  %\VignetteIndexEntry{Detecting separation and infinite estimates in log binomial regression}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width = 6,
  fig.height = 6
)
```


# Introduction
In contrast to logistic regression in log-binomial regression, separation is not
the only factor which decides if the maximum likelihood estimate (MLE) has
infinite components.
As we will see in the example below, for the log-binomial regression model (LBRM)
there exist data configurations which are separated but the MLE still exists.

The MLE of the LBRM can be obtained by solving the following optimization problem.
$$
\begin{equation}
    \begin{array}{rl}
    \underset{\beta}{\textrm{maximize}} &
    \ell(\beta) = \displaystyle\sum_{i = 1}^n y_i ~ X_{i*} \beta + (1 - y_i) ~  \log(1 - \exp(X_{i*} \beta)) \ \ \ \
    \textrm{s.t.} &
    X \beta \leq 0.
    \end{array}
\end{equation}
$$

From the optimization problem we can already guess that the different behavior 
with regard to the existence of the MLE is caused by the linear constraint
$X \beta \leq 0$.


Let $X^0$ be the submatrix of $X$ obtained by keeping only the rows
$I^0 = \{i|y_i = 0\}$ and $X^1$ the submatrix obtained by keeping only
the rows $I^1 = \{i|y_i = 1\}$.
@schwendinger+gruen+hornik:2021 pointed out that the finiteness of the MLE 
can be checked by solving the following linear optimization problem.

$$
\begin{equation}
\begin{array}{rll}
  \underset{\beta}{\text{maximize}}~~  &
    - \sum_{i \in I^0} X_{i*} \beta \\
  \text{subject to}~~  &
    X^0 \beta \leq 0 \\
  & X^1 \beta = 0.
\end{array}
\end{equation}
$$
The MLE has only finite components if the solution of this linear program is a
zero vector. If the MLE contains infinite components, the linear programming problem
is unbounded. The function `detect_infinite_estimates()` from the **detectseparation** 
implements the LP problem described above and can therefore be used to detect
infinite components in the MLE of the LBRM. 

```{r, echo = TRUE, eval = TRUE}
library("detectseparation")
```

---

# Example
To show the different effect of separation on the logistic regression model
compared to the LBRM consider the following data.
```{r, tiny_example}
data <- data.frame(a = c(1, 0, 3, 2, 3, 4),
                   b = c(2, 1, 1, 4, 6, 8),
                   y = c(0, 0, 0, 1, 1, 1))
```

## Detect separation

Clearly the data is separated, which can be verified by using the `detect_separation`
method (a detailed explanation of the output can be found in [Section 3](#output_details)).

```{r, tiny_example_sep}
glm(y ~ a + b, data = data, family = binomial("logit"), method = "detect_separation")
```

Since separation is a property of the data, checking for separation gives the
same result for the logistic regression and the LBRM.

```{r, tiny_example_sep_lbrm}
glm(y ~ a + b, data = data, family = binomial("log"), method = "detect_separation")
```

## Detect infinite estimates

For logistic regression separation is necessary and sufficient that the MLE 
contains infinite components.

```{r, tiny_example_inf_est}
glm(y ~ a + b, data = data, family = binomial("logit"), method = "detect_infinite_estimates")
```

However, due to the linear constraint of the LBRM, there exists data allocations where
the MLE does exist (i.e., has only finite components), despite the fact that
the data is separated.

```{r}
glm(y ~ a + b, data = data, family = binomial("log"), method = "detect_infinite_estimates")
```

## Fitting the LBRM

Using `glm` to solve this problem we get the following error message.

```{r, glm_fit}
fit <- try(glm(y ~ a + b, data = data, family = binomial("log")))
```

The error message means that we should provide starting values, a simple
but reliable approach is to use $(-1, 0, ..., 0)$ as starting value.

Since in this example one of the constraints $X_{i*} \beta \leq 0$
is by design binding, the iteratively re-weighted least squares (IRLS)
method used by the `glm` function has convergence problems.
The `glm` function informs us about the convergence problems by
issuing some warnings. The warnings 

```
#> Warning: step size truncated: out of bounds
#> Warning: glm.fit: algorithm stopped at boundary value
```

tell us that IRLS is not the best option for optimization problems with binding constraints.
The warning 

```
#> Warning: glm.fit: algorithm did not converge
```

tell us that the algorithm did not converge. Practically in most cases this just means
the default value for the maximum number of iterations should be increased.


```{r}
args(glm.control)
```
Since the default value for the maximum number of iterations is quite low (`maxit = 25`).
However, `maxit = 25` is typically high enough for unbounded optimization problems 
(almost all models supported by `glm`), but is often to low for the LBRM.
For most data sets setting `maxit = 10000` when estimating LBRMs is high enough.

```{r, tiny_example_inf_esti_log_separation}
formula <- y ~ a + b
start <- c(-1, double(ncol(model.matrix(formula, data = data)) - 1L))
ctrl = glm.control(epsilon = 1e-8, maxit = 10000, trace = FALSE)
suppressWarnings(
  fit <- glm(formula, data = data, family = binomial("log"), start = start, control = ctrl)
)
summary(fit)
```

We can verify that one of the constraints $X_{i*} \beta \leq 0$ is binding
(i.e., $X_{i*} \beta = 0$ for at least one $i$) by multiplying the
coefficients with the model matrix.
```{r}
print(mm <- drop(model.matrix(formula, data) %*% coef(fit)))
abs(drop(mm)) < 1e-6
```

---

# Details on the output{#output_details}

## Detect separation

```{r, explain_output_detect_separation}
glm(y ~ a + b, data = data, family = binomial("logit"), method = "detect_separation")
```

The output above provides much information:

1. The output shows, that for the verification oft the separation the linear optimization solver **lpsolve** was used.  
2. The line `Separation: TRUE` indicates that the data is separated.
3. The coefficients at `Inf` and `-Inf` indicate that the underlying optimization problem is unbounded and therefore the MLE does not exist for the logistic regression model.

## Detect infinite estimates

```{r, explain_output_detect_infinite_estimates}
glm(y ~ a + b, data = data, family = binomial("log"), method = "detect_infinite_estimates")
```

The output above provides much information:

1. The output shows, that for the verification oft the separation the linear optimization solver **lpsolve** was used.  
2. The line `Infinite estimates: FALSE` indicates that the MLE has only finite components (the MLE exists).
3. The coefficients are all `0` which again indicates that the MLE has only finite components
and therefore underlying optimization problem is bounded.

---

# Choosing starting values
For the LBRM the `glm` function often requires the users to specify starting values.
Valid starting values have to reside in the interior of the feasible region ($X \beta < 0$).
There have been different methods suggested for finding valid starting values.

## Simple approach
If an intercept is present in the estimation `(-1, 0, ..., 0)` will always provide 
valid starting values.
```{r}
find_start_simple <- function(formula, data) {
  c(-1, double(ncol(model.matrix(formula, data = data)) - 1L))  
}

find_start_simple(formula, data)
max(model.matrix(formula, data = data) %*% find_start_simple(formula, data))
```

## Hot start via Poisson model
@andrade+andrade2018 suggest a hot start method, by using the modified estimation result
of a Poisson model with log link as starting values. Again this method only works if an
intercept is used.
```{r}
find_start_poisson <- function(formula, data, delta = 1) {
  b0 <- coef(glm(formula, data, family = poisson(link = "log")))
  mX <- -model.matrix(formula, data = data)[, -1L, drop = FALSE]
  b0[1] <- min(mX %*% b0[-1]) - delta
  b0
}

find_start_poisson(formula, data)
max(model.matrix(formula, data = data) %*% find_start_poisson(formula, data))
```

## Recommendation
One can also solve an LP to find valid start values or think of other strategies.
However, for the benchmark examples reported in @schwendinger+gruen+hornik:2021
we found no conclusive evidence that one of these initialization methods outperforms
the others. Therefore, my personal favorite is the simple approach `(-1, 0, ..., 0)`.


---

# References

---
title: "Detect/check for separation and infinite maximum likelihood estimates in logistic regression"
author: "[Ioannis Kosmidis](https://www.ikosmidis.com) and [Dirk Schumacher](https://www.dirk-schumacher.net)"
date: "5 January 2020"
output: rmarkdown::html_vignette
bibliography: detectseparation.bib
vignette: >
  %\VignetteIndexEntry{Detecting separation and infinite estimates in logistic regression}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width = 6,
  fig.height = 6
)
```


# The **detectseparation** package

[**detectseparation**](https://github.com/ikosmidis/detectseparation)
provides *pre-fit* and *post-fit* methods for the detection of
separation and of infinite maximum likelihood estimates in binomial
response generalized linear models.

The key methods are `detect_separation` and `check_infinite_estimates`
and this vignettes describes their use.

# Checking for infinite estimates

@heinze+schemper:2002 used a logistic regression model to analyze data
from a study on endometrial cancer [see, @agresti:2015, Section 5.7 or
`?endometrial` for more details on the data set]. Below, we refit the
model in @heinze+schemper:2002 in order to demonstrate the
functionality that **detectseparation** provides.

```{r, echo = TRUE, eval = TRUE}
library("detectseparation")
data("endometrial", package = "detectseparation")
endo_glm <- glm(HG ~ NV + PI + EH, family = binomial(), data = endometrial)
theta_mle <- coef(endo_glm)
summary(endo_glm)
```

The maximum likelihood (ML) estimate of the parameter for `NV` is actually
infinite. The reported, apparently finite value is merely due to false
convergence of the iterative estimation procedure. The same is true
for the estimated standard error, and, hence the value `r
round(coef(summary(endo_glm))["NV", "z value"], 3)` for the $z$-statistic
cannot be trusted for inference on the size of the effect for `NV`.

@lesaffre+albert:1989[, Section 4] describe a procedure that can hint
on the occurrence of infinite estimates. In particular, the model is
successively refitted, by increasing the maximum number of allowed
iteratively re-weighted least squares iterations at east step. The
estimated asymptotic standard errors from each step are, then, divided
to the corresponding ones from the first fit. If the sequence of
ratios diverges, then the maximum likelihood estimate of the
corresponding parameter is minus or plus infinity. The following code
chunk applies this process to `endo_glm`.

```{r, echo = TRUE, eval = TRUE }
(inf_check <- check_infinite_estimates(endo_glm))
plot(inf_check)
```

Clearly, the ratios of estimated standard errors diverge for `NV`.

# Detecting separation

`detect_separation` tests for the occurrence of complete or
quasi-complete separation in datasets for binomial response
generalized linear models, and finds which of the parameters will have
infinite maximum likelihood estimates. `detect_separation` relies on
the linear programming methods developed in the 2017 PhD thesis by
Kjell Konis [@konis:2007].

`detect_separation` is *pre-fit* method, in the sense that it does not
need to estimate the model to detect separation and/or identify
infinite estimates. For example

```{r, endo_sep, eval = TRUE, echo = TRUE}
endo_sep <- glm(HG ~ NV + PI + EH, data = endometrial,
                family = binomial("logit"),
                method = "detect_separation")
endo_sep
```

The `detect_separation` method reports that there is separation in the
data, that the estimates for `(Intercept)`, `PI` and `EH` are finite
(coded 0), and that the estimate for `NV` is plus infinity. So, the
actual maximum likelihood estimates are

```{r, echo = TRUE,  eval = TRUE}
coef(endo_glm) + coef(endo_sep)
```

and the estimated standard errors are

```{r, echo = TRUE,  eval = TRUE}
coef(summary(endo_glm))[, "Std. Error"] + abs(coef(endo_sep))
```

We can also use the
[`glpk`](https://CRAN.R-project.org/package=ROI.plugin.glpk) solver
for solving the linear program for separation detection

```{r, echo = TRUE, eval = TRUE}
update(endo_sep, solver = "glpk")
```

or use the implementation using
[**lpSolveAPI**](https://CRAN.R-project.org/package=ROI.plugin.glpk)
directly

```{r,  echo = TRUE, eval = TRUE}
update(endo_sep, implementation = "lpSolveAPI")
```
See `?detect_separation_control` for more options.

As proven in [@kosmidis+firth:2021], an estimator that is always
finite, regardless whether separation occurs on not, is the
reduced-bias estimator of [@firth:1993], which is implemented in the
[**brglm2**](https://CRAN.R-project.org/package=brglm2) R package.

```{r, echo = TRUE, eval = TRUE}
library("brglm2")
summary(update(endo_glm, method = "brglm_fit"))
```

# Citation
If you found this vignette or **detectseparation** useful, please
consider citing **detectseparation**. You can find information on how
to do this by typing `citation("detectseparation")`.

# References

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{silvapulle1981}
\alias{silvapulle1981}
\title{Separation Example Presented in Silvapulle (1981)}
\format{
A data frame with 35 rows and 2 variables:
\itemize{
\item \code{y}: a factor with the levels \code{case} and \code{none-case}, giving the
outcome of a standardized psychiatric interview
\item \code{ghqs}: an integer giving the general health questionnaire score.
}
}
\usage{
silvapulle1981
}
\description{
Separation example presented in Silvapulle (1981).
}
\references{
Silvapulle, M. J. (1981).
On the Existence of Maximum Likelihood Estimators for the Binomial Response Models.
Journal of the Royal Statistical Society. Series B (Methodological), 43(3), 310–313.
\url{https://www.jstor.org/stable/2984941}
}
\seealso{
\code{\link[=detect_infinite_estimates]{detect_infinite_estimates()}}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{endometrial}
\alias{endometrial}
\title{Histology grade and risk factors for 79 cases of endometrial cancer}
\format{
A data frame with 79 rows and 4 variables:
\itemize{
\item \code{NV}: neovasculization with coding 0 for absent and 1 for present
\item \code{PI}: pulsality index of arteria uterina
\item \code{EH}: endometrium height
\item \code{HG}: histology grade with coding 0 for low grade and 1 for high grade
}
}
\source{
The packaged data set was downloaded in \code{.dat} format from
\url{https://users.stat.ufl.edu/~aa/glm/data/}. The latter link
provides the data sets used in Agresti (2015).

The endometrial data set was first analyzed in Heinze and
Schemper (2002), and was originally provided by Dr
E. Asseryanis from the Medical University of Vienna.
}
\usage{
endometrial
}
\description{
Histology grade and risk factors for 79 cases of endometrial cancer
}
\references{
Agresti A (2015). \emph{Foundations of Linear and Generalized Linear
Models}.  Wiley Series in Probability and Statistics. Wiley.

Heinze G, Schemper M (2002). A Solution to the Problem of
Separation in Logistic Regression. \emph{Statistics in Medicine},
\strong{21}, 2409–2419. \doi{10.1002/sim.1047}.

Kosmidis I, Firth D (2021). Jeffreys-prior penalty, finiteness
and shrinkage in binomial-response generalized linear
models. \emph{Biometrika}, \strong{108}, 71-82. \doi{10.1093/biomet/asaa052}.
}
\seealso{
\code{\link[=detect_separation]{detect_separation()}}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/detect_separation.R, R/zzz_conventions.R
\name{detect_separation_control}
\alias{detect_separation_control}
\alias{detectSeparationControl}
\title{Auxiliary function for the \code{\link[=glm]{glm()}} interface when
\code{method} is \code{\link[=detect_separation]{detect_separation()}}}
\usage{
detect_separation_control(
  implementation = c("ROI", "lpSolveAPI"),
  solver = "lpsolve",
  linear_program = c("primal", "dual"),
  purpose = c("find", "test"),
  tolerance = 1e-04,
  solver_control = list(),
  separation_type = FALSE
)

detectSeparationControl(
  implementation = c("ROI", "lpSolveAPI"),
  solver = "lpsolve",
  linear_program = c("primal", "dual"),
  purpose = c("find", "test"),
  tolerance = 1e-04,
  solver_control = list(),
  separation_type = FALSE
)
}
\arguments{
\item{implementation}{should the implementation using \code{ROI} or the
implementation using \code{lpSolveAPI} be used? Default is \code{ROI}.}

\item{solver}{should the linear program be solved using the
\code{"lpsolve"} (using the \code{ROI.plugin.lpsolve} package; default)
or another solver? Alternative solvers are \code{"glpk"}, \code{"cbc"},
\code{"clp"}, \code{"cplex"}, \code{"ecos"}, \code{"gurobi"}, \code{"scs"},
\code{"symphony"}. If the corresponding package
\code{ROI.plugin.[solver]} is not installed then the user will be
prompted to install it before continuing.}

\item{linear_program}{should \code{\link[=detect_separation]{detect_separation()}} solve the
\code{"primal"} (default) or \code{"dual"} linear program for separation
detection? Only relevant if \code{implementation = "lpSolveAPI"}.}

\item{purpose}{should \code{\link[=detect_separation]{detect_separation()}} simply \code{"test"} for
separation or also \code{"find"} (default) which parameters are
infinite? Only relevant if \code{implementation = "lpSolveAPI"}.}

\item{tolerance}{maximum absolute variable value from the linear
program, before separation is declared. Default is \code{1e-04}.}

\item{solver_control}{a list with additional control parameters for
the \code{"solver"}. This is solver specific, so consult the
corresponding documentation. Default is \code{list()} unless
\code{solver} is \code{"alabama"} when the default is \code{list(start = rep(0, p))}, where \code{p} is the number of parameters.}

\item{separation_type}{logical. Should \code{\link[=detect_separation]{detect_separation()}}
attempt to distinguish complete from quasi-complete separation
after separation has been detected? If \code{TRUE} and separation is
detected, then an additional linear program is solved. Default
is \code{FALSE}.}
}
\value{
A list with the supplied \code{linear_program}, \code{solver},
\code{solver_control}, \code{purpose}, \code{tolerance}, \code{separation_type},
and \code{implementation}. The returned list is intended to be passed to
the \code{control} argument of \code{\link[=detect_separation]{detect_separation()}} and
\code{\link[=detect_infinite_estimates]{detect_infinite_estimates()}}.
}
\description{
Typically only used internally by \code{\link[=detect_separation]{detect_separation()}}
but may be used to construct a \code{control} argument.
}
\examples{
data("endometrial", package = "detectseparation")
ctrl <- detect_separation_control(separation_type = TRUE)
glm(HG ~ NV + PI + EH, data = endometrial,
    family = binomial("logit"),
    method = "detect_separation",
    control = ctrl)

}
\references{
Konis K. (2007). \emph{Linear Programming Algorithms for Detecting
Separated Data in Binary Logistic Regression Models}. DPhil.
University of Oxford.
\url{https://ora.ox.ac.uk/objects/uuid:8f9ee0d0-d78e-4101-9ab4-f9cbceed2a2a}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/detectseparation-package.R
\docType{package}
\name{detectseparation}
\alias{detectseparation-package}
\alias{detectseparation}
\title{detectseparation: Methods for Detecting and Checking for Separation
and Infinite Maximum Likelihood Estimates}
\description{
\pkg{detectseparation} provides pre-fit and post-fit methods for
the detection of separation and of infinite maximum likelihood
estimates in binomial response generalized linear models.
}
\details{
The key methods are \code{\link[=detect_separation]{detect_separation()}} and
\code{\link[=check_infinite_estimates]{check_infinite_estimates()}}.
}
\seealso{
\code{\link[=detect_separation]{detect_separation()}}, \code{\link[=check_infinite_estimates]{check_infinite_estimates()}}
}
\author{
\strong{Maintainer}: Ioannis Kosmidis \email{ioannis.kosmidis@warwick.ac.uk} (\href{https://orcid.org/0000-0003-1556-0302}{ORCID})

Authors:
\itemize{
  \item Dirk Schumacher \email{mail@dirk-schumacher.net}
  \item Florian Schwendinger \email{FlorianSchwendinger@gmx.at}
}

Other contributors:
\itemize{
  \item Kjell Konis \email{kjell.konis@me.com} [contributor]
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/detect_separation.R, R/zzz_conventions.R
\name{detect_separation}
\alias{detect_separation}
\alias{detectSeparation}
\alias{print.detect_separation}
\title{Detect Separation}
\usage{
detect_separation(
  x,
  y,
  weights = NULL,
  start = NULL,
  etastart = NULL,
  mustart = NULL,
  offset = NULL,
  family = gaussian(),
  control = list(),
  intercept = TRUE,
  singular.ok = TRUE
)

detectSeparation(
  x,
  y,
  weights = NULL,
  start = NULL,
  etastart = NULL,
  mustart = NULL,
  offset = NULL,
  family = gaussian(),
  control = list(),
  intercept = TRUE,
  singular.ok = TRUE
)
}
\arguments{
\item{x}{\code{x} is a design matrix of dimension \code{n * p}.}

\item{y}{\code{y} is a vector of observations of length \code{n}.}

\item{weights}{an optional vector of \sQuote{prior weights} to be used
    in the fitting process.  Should be \code{NULL} or a numeric vector.}

\item{start}{currently not used.}

\item{etastart}{currently not used.}

\item{mustart}{currently not used.}

\item{offset}{this can be used to specify an \emph{a priori} known
    component to be included in the linear predictor during fitting.
    This should be \code{NULL} or a numeric vector of length equal to
    the number of cases.  One or more \code{\link[stats]{offset}} terms can be
    included in the formula instead or as well, and if more than one is
    specified their sum is used.  See \code{\link[stats]{model.offset}}.}

\item{family}{a description of the error distribution and link
    function to be used in the model.  For \code{glm} this can be a
    character string naming a family function, a family function or the
    result of a call to a family function.  For \code{glm.fit} only the
    third option is supported.  (See \code{\link[stats]{family}} for details of
    family functions.)}

\item{control}{a list of parameters controlling separation
detection. See \code{detect_separation_control()} for
details.}

\item{intercept}{logical. Should an intercept be included in the
    \emph{null} model?}

\item{singular.ok}{logical. If \code{FALSE}, a singular model is an
error.}
}
\value{
A list that inherits from class \code{\link{detect_separation}},
\code{\link{glm}} and \code{\link{lm}}. A \code{print} method is provided for
\code{\link{detect_separation}} objects. If
\code{detect_separation_control(separation_type = TRUE)} is used and
separation is detected, then the returned object has a \code{complete}
component, which is \code{TRUE} for complete separation and \code{FALSE} for
quasi-complete separation. Otherwise, the \code{complete} component is
\code{NULL}.
}
\description{
Method for \code{\link[=glm]{glm()}} that tests for data separation and
finds which parameters have infinite maximum likelihood estimates
in generalized linear models with binomial responses

\code{\link[=detect_separation]{detect_separation()}} is a method for \code{\link[=glm]{glm()}}
that tests for the occurrence of complete or quasi-complete
separation in datasets for binomial response generalized linear
models, and finds which of the parameters will have infinite
maximum likelihood estimates. \code{\link[=detect_separation]{detect_separation()}}
relies on the linear programming methods developed in Konis (2007).
}
\details{
Following the definitions in Albert and Anderson (1984), the data
for a binomial-response generalized linear model with logistic link
exhibit quasi-complete separation if there exists a non-zero
parameter vector \eqn{\beta} such that \eqn{X^0 \beta \le 0} and
\eqn{X^1 \beta \ge 0}, where \eqn{X^0} and \eqn{X^1} are the
matrices formed by the rows of the model matrix $X$ corresponding
to zero and non-zero responses, respectively. The data exhibits
complete separation if there exists a parameter vector \eqn{\beta} such
that the aforementioned conditions are satisfied with strict
inequalities. If there are no vectors \eqn{\beta} that can satisfy the
conditions, then the data points are said to overlap.

If the inverse link function \eqn{G(t)} of a generalized linear
model with binomial responses is such that \eqn{\log G(t)} and
\eqn{\log (1 - G(t))} are concave and the model has an intercept
parameter, then overlap is a necessary and sufficient condition for
the maximum likelihood estimates to be finite (see Silvapulle, 1981
for a proof). Such link functions are, for example, the logit,
probit and complementary log-log.

\code{\link[=detect_separation]{detect_separation()}} determines whether or not the
data exhibits (quasi-)complete separation. Then, if separation is
detected and the link function \eqn{G(t)} is such that \eqn{\log
G(t)} and \eqn{\log (1 - G(t))} are concave, the maximum likelihood
estimates has infinite components.

\code{\link[=detect_separation]{detect_separation()}} is a wrapper to the
\code{\link[=detect_infinite_estimates]{detect_infinite_estimates()}} method. Separation
detection, as separation is defined above, takes place using the
linear programming methods in Konis (2007) regardless of the link
function. The output of those methods is also used to determine
which estimates are infinite, unless the link is \code{"log"}. In the
latter case the linear programming methods in Schwendinger et
al. (2021) are called to establish if and which estimates are
infinite. If the link function is not one of \code{"logit"}, \code{"log"},
\code{"probit"}, \code{"cauchit"}, \code{"cloglog"} then a warning is issued.

If \code{separation_type = TRUE} in
\code{detect_separation_control()}, then, whenever
separation is detected, \code{\link[=detect_separation]{detect_separation()}} attempts
to distinguish between complete and quasi-complete separation by
solving an additional linear program that maximizes the minimum
transformed margin. If \eqn{x_i} is the \eqn{i}th row of the model
matrix and \eqn{\tilde{y}_i = -1 + 2 y_i}, where \eqn{y_i} is the
\eqn{i}th Bernoulli response (the representation to which any
binomial data supplied by the user are transformed internally),
then \eqn{\bar{x}_i = \tilde{y}_i x_i}. The additional linear
program that is solved maximizes \eqn{t} subject to
\eqn{\bar{X}\beta \ge t {\bf 1}} and \eqn{-1 \le \beta_j \le
1}. This is equivalent to solving \eqn{\max_\beta \min_i
(\bar{x}_i^\top \beta)} subject to \eqn{-1 \le \beta_j \le 1} for
all \eqn{j}. A positive optimal value for \eqn{t} implies complete
separation, while an optimal value of zero implies
quasi-complete separation provided that separation has already been
detected. See Konis (2007, Section 1.3 and Chapter 4) for the
definitions of separation and the transformed linear programming
formulation.

The \code{\link[=coefficients]{coefficients()}} method extracts a vector of values
for each of the model parameters under the following convention:
\code{0} if the maximum likelihood estimate of the parameter is
finite, and \code{Inf} or \code{-Inf} if the maximum likelihood
estimate of the parameter is plus or minus infinity. This
convention makes it easy to adjust the maximum likelihood estimates
to their actual values by element-wise addition.

\code{\link[=detect_separation]{detect_separation()}} can be passed directly as
a method to the \code{\link[=glm]{glm()}} function. See, examples.

\code{\link[=detectSeparation]{detectSeparation()}} is an alias for \code{\link[=detect_separation]{detect_separation()}}.
}
\note{
For the definition of complete and quasi-complete separation, see
Albert and Anderson (1984). Kosmidis and Firth (2021) prove that
the reduced-bias estimator that results by the penalization of the
logistic regression log-likelihood by Jeffreys prior takes always
finite values, even when some of the maximum likelihood estimates
are infinite. The reduced-bias estimates can be computed using the
\pkg{brglm2} R package.

\code{\link[=detect_separation]{detect_separation()}} was designed in 2017 by Ioannis
Kosmidis for the \pkg{brglm2} R package, after correspondence with
Kjell Konis, and a port of the \code{separator()} function had been
included in \pkg{brglm2} under the permission of Kjell Konis. In
2020, \code{\link[=detect_separation]{detect_separation()}} and
\code{\link[=check_infinite_estimates]{check_infinite_estimates()}} were moved outside
\strong{brglm2} into the dedicated \strong{detectseparation} package. Dirk
Schumacher authored the \code{separator_ROI()} function, which
depends on the \pkg{ROI} R package and is now the default
implementation used for detecting separation. In 2022, Florian
Schwendinger authored the \code{dielb_ROI()} function for detecting
infinite estimates in log-binomial regression, and, with Ioannis
Kosmidis, they refactored the codebase to properly accommodate for
the support of log-binomial regression.
}
\examples{

# endometrial data from Heinze \& Schemper (2002) (see ?endometrial)
data("endometrial", package = "detectseparation")
endometrial_sep <- glm(HG ~ NV + PI + EH, data = endometrial,
                       family = binomial("logit"),
                       method = "detect_separation")
endometrial_sep
# The maximum likelihood estimate for NV is infinite
summary(update(endometrial_sep, method = "glm.fit"))

# If we want to further check for the type of separation (complete
# or quasi-complete) we can do
endometrial_sep_type <- update(endometrial_sep, separation_type = TRUE)
endometrial_sep
endometrial_sep_type

\donttest{
# Example inspired by unpublished microeconometrics lecture notes by
# Achim Zeileis https://eeecon.uibk.ac.at/~zeileis/
# The maximum likelihood estimate of sourhernyes is infinite
if (requireNamespace("AER", quietly = TRUE)) {
    data("MurderRates", package = "AER")
    murder_sep <- glm(I(executions > 0) ~ time + income +
                      noncauc + lfp + southern, data = MurderRates,
                      family = binomial(), method = "detect_separation")
    murder_sep
    # which is also evident by the large estimated standard error for NV
    murder_glm <- update(murder_sep, method = "glm.fit")
    summary(murder_glm)
    # and is also revealed by the divergence of the NV column of the
    # result from the more computationally intensive check
    plot(check_infinite_estimates(murder_glm))
    # Mean bias reduction via adjusted scores results in finite estimates
    if (requireNamespace("brglm2", quietly = TRUE))
        update(murder_glm, method = brglm2::brglm_fit)
}
}
}
\references{
Konis K. (2007). \emph{Linear Programming Algorithms for Detecting
Separated Data in Binary Logistic Regression
Models}. DPhil. University of Oxford.
\url{https://ora.ox.ac.uk/objects/uuid:8f9ee0d0-d78e-4101-9ab4-f9cbceed2a2a}

Konis K. (2013). safeBinaryRegression: Safe Binary Regression. R
package version 0.1-3.
\url{https://CRAN.R-project.org/package=safeBinaryRegression}

Kosmidis I. and Firth D. (2021). Jeffreys-prior penalty, finiteness
and shrinkage in binomial-response generalized linear
models. \emph{Biometrika}, \strong{108}, 71–82. \doi{10.1093/biomet/asaa052}

Silvapulle, M. J. (1981).  On the Existence of Maximum Likelihood
Estimators for the Binomial Response Models.  \emph{Journal of the Royal
Statistical Society. Series B (Methodological)}, \strong{43}, 310–313.
\url{https://www.jstor.org/stable/2984941}

Schwendinger, F., Grün, B. & Hornik, K. (2021). A comparison of
optimization solvers for log binomial regression including conic
programming.  \emph{Computational Statistics}, \strong{36},
1721–1754. \doi{10.1007/s00180-021-01084-5}
}
\seealso{
\code{\link[=glm]{glm()}}, \code{\link[=detect_infinite_estimates]{detect_infinite_estimates()}}, \code{\link[=check_infinite_estimates]{check_infinite_estimates()}}, \code{\link[brglm2:brglmFit]{brglm2::brglm_fit()}}
}
\author{
Ioannis Kosmidis [aut, cre] \email{ioannis.kosmidis@warwick.ac.uk}, Dirk Schumacher [aut] \email{mail@dirk-schumacher.net}, Florian Schwendinger [aut] \email{FlorianSchwendinger@gmx.at}, Kjell Konis [ctb] \email{kjell.konis@me.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/detect_infinite_estimates.R,
%   R/zzz_conventions.R
\name{detect_infinite_estimates}
\alias{detect_infinite_estimates}
\alias{detectInfiniteEstimates}
\title{Detect Infinite Estimates}
\usage{
detect_infinite_estimates(
  x,
  y,
  weights = NULL,
  start = NULL,
  etastart = NULL,
  mustart = NULL,
  offset = NULL,
  family = gaussian(),
  control = list(),
  intercept = TRUE,
  singular.ok = TRUE
)

detectInfiniteEstimates(
  x,
  y,
  weights = NULL,
  start = NULL,
  etastart = NULL,
  mustart = NULL,
  offset = NULL,
  family = gaussian(),
  control = list(),
  intercept = TRUE,
  singular.ok = TRUE
)
}
\arguments{
\item{x}{\code{x} is a design matrix of dimension \code{n * p}.}

\item{y}{\code{y} is a vector of observations of length \code{n}.}

\item{weights}{an optional vector of \sQuote{prior weights} to be used
    in the fitting process.  Should be \code{NULL} or a numeric vector.}

\item{start}{currently not used.}

\item{etastart}{currently not used.}

\item{mustart}{currently not used.}

\item{offset}{this can be used to specify an \emph{a priori} known
    component to be included in the linear predictor during fitting.
    This should be \code{NULL} or a numeric vector of length equal to
    the number of cases.  One or more \code{\link[stats]{offset}} terms can be
    included in the formula instead or as well, and if more than one is
    specified their sum is used.  See \code{\link[stats]{model.offset}}.}

\item{family}{a description of the error distribution and link
    function to be used in the model.  For \code{glm} this can be a
    character string naming a family function, a family function or the
    result of a call to a family function.  For \code{glm.fit} only the
    third option is supported.  (See \code{\link[stats]{family}} for details of
    family functions.)}

\item{control}{a list of parameters controlling separation
detection. See \code{\link[=detect_separation_control]{detect_separation_control()}} for
details.}

\item{intercept}{logical. Should an intercept be included in the
    \emph{null} model?}

\item{singular.ok}{logical. If \code{FALSE}, a singular model is an
error.}
}
\description{
Method for \code{\link[=glm]{glm()}} that detects infinite components in
the maximum likelihood estimates of generalized linear models
with binomial responses.
}
\details{
For binomial-response generalized linear models with \code{"log"} link,
separated data allocations do not necessarily lead to infinite
maximum likelihood estimates. For this reason, for models with the
\code{"log"} link \code{\link[=detect_infinite_estimates]{detect_infinite_estimates()}} relies on an alternative
linear optimization model developed in Schwendinger et
al. (2021). For all the other supported links
\code{\link[=detect_infinite_estimates]{detect_infinite_estimates()}} relies on the linear programming
methods developed in Konis (2007). See \code{\link[=detect_separation]{detect_separation()}} for
definitions and details.

\code{\link[=detect_infinite_estimates]{detect_infinite_estimates()}} is a wrapper to the
functions \code{separator_ROI()}, \code{separator_lpSolveAPI()} (a
modified version of the \code{separator()} function from the
\pkg{safeBinaryRegression} R package), and \code{dielb_ROI()}. The first
two are used for non-log links, while \code{dielb_ROI()} is used for the
\code{"log"} link.

The \code{\link[=coefficients]{coefficients()}} method extracts a vector of values
for each of the model parameters under the following convention:
\code{0} if the maximum likelihood estimate of the parameter is
finite, and \code{Inf} or \code{-Inf} if the maximum likelihood
estimate of the parameter is plus or minus infinity. This
convention makes it easy to adjust the maximum likelihood estimates
to their actual values by element-wise addition.

\code{\link[=detect_infinite_estimates]{detect_infinite_estimates()}} can be passed directly as
a method to the \code{\link[=glm]{glm()}} function. See, examples.

\code{\link[=detectInfiniteEstimates]{detectInfiniteEstimates()}} is an alias for
\code{\link[=detect_infinite_estimates]{detect_infinite_estimates()}}.
}
\examples{
# The classical example given in Silvapulle (1981) can be utilized
# to show that for the log-binomial model there exist data allocations
# which are separated but produce finite estimates.
data("silvapulle1981", package = "detectseparation")

# Since the data is separated the MLE does not exist for the logit link.
glm(y ~ ghqs, data = silvapulle1981, family = binomial(),
    method = "detect_infinite_estimates")

# However, for the log link all components of the MLE are finite.
glm(y ~ ghqs, data = silvapulle1981, family = binomial("log"),
    method = "detect_infinite_estimates")
# The maximum likelihood fit is
glm(y ~ ghqs, data = silvapulle1981, family = binomial("log"), start = c(-1, 0))

}
\references{
Silvapulle, M. J. (1981).
On the Existence of Maximum Likelihood Estimators for the Binomial Response Models.
Journal of the Royal Statistical Society. Series B (Methodological), 43(3), 310–313.
\url{https://www.jstor.org/stable/2984941}

Konis K. (2007). \emph{Linear Programming Algorithms for Detecting
Separated Data in Binary Logistic Regression
Models}. DPhil. University of Oxford.
\url{https://ora.ox.ac.uk/objects/uuid:8f9ee0d0-d78e-4101-9ab4-f9cbceed2a2a}

Konis K. (2013). safeBinaryRegression: Safe Binary Regression. R
package version 0.1-3.
\url{https://CRAN.R-project.org/package=safeBinaryRegression}

Kosmidis I. and Firth D. (2021). Jeffreys-prior penalty, finiteness
and shrinkage in binomial-response generalized linear
models. \emph{Biometrika}, \strong{108}, 71–82. \doi{10.1093/biomet/asaa052}

Schwendinger, F., Grün, B. & Hornik, K. (2021). A comparison of
optimization solvers for log binomial regression including conic
programming.  \emph{Computational Statistics}, \strong{36},
1721–1754. \doi{10.1007/s00180-021-01084-5}
}
\seealso{
\code{\link[=glm]{glm()}}, \code{\link[=detect_separation]{detect_separation()}}, \code{\link[=check_infinite_estimates]{check_infinite_estimates()}}, \code{\link[brglm2:brglmFit]{brglm2::brglm_fit()}}
}
\author{
Ioannis Kosmidis [aut, cre] \email{ioannis.kosmidis@warwick.ac.uk}, Florian Schwendinger [aut] \email{FlorianSchwendinger@gmx.at}, Dirk Schumacher [aut] \email{mail@dirk-schumacher.net}, Kjell Konis [ctb] \email{kjell.konis@me.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/check_infinite_estimates.R
\name{check_infinite_estimates.glm}
\alias{check_infinite_estimates.glm}
\title{A simple diagnostic of whether the maximum likelihood estimates are
infinite}
\usage{
\method{check_infinite_estimates}{glm}(object, nsteps = 20, ...)
}
\arguments{
\item{object}{the result of a \code{\link[=glm]{glm()}} call.}

\item{nsteps}{starting from \code{maxit = 1}, the GLM is refitted
for \code{maxit = 2}, \code{maxit = 3}, \ldots, \code{maxit = nsteps}. Default value is \code{20}.}

\item{...}{currently not used.}
}
\value{
An object of class \code{inf_check} that has a \code{\link[=plot]{plot()}} method.

A matrix inheriting from class \code{inf_check}, with \code{nsteps}
rows and \code{p} columns, where \code{p} is the number of model
parameters. A \code{\link[=plot]{plot()}} method is provided for \code{inf_check}
objects for the easy inspection of the ratios of the standard
errors.
}
\description{
A simple diagnostic of whether the maximum likelihood estimates are
infinite
}
\details{
\code{\link[=check_infinite_estimates]{check_infinite_estimates()}} attempts to identify the occurrence
of infinite estimates in GLMs with binomial responses by
successively refitting the model. At each iteration the maximum
number of allowed IWLS iterations is fixed starting from 1 to
\code{nsteps} (by setting \code{control = glm.control(maxit = j)},
where \code{j} takes values \verb{1, ..., nsteps} in
\code{\link[=glm]{glm()}}. For each value of \code{maxit}, the estimated
asymptotic standard errors are divided to the corresponding ones
from \code{control = glm.control(maxit = 1)}. Then, based on the
results in Lesaffre & Albert (1989), if the sequence of ratios in
any column of the resultant matrix diverges, then complete or
quasi-complete separation occurs and the maximum likelihood
estimate for the corresponding parameter has value minus or plus
infinity.

\code{\link[=check_infinite_estimates]{check_infinite_estimates()}} can also be used to identify the
occurrence of infinite estimates in baseline category logit models
for nominal responses (see \code{\link[brglm2:brmultinom]{brglm2::brmultinom()}}), and adjacent category
logit models for ordinal responses (see \code{\link[brglm2:bracl]{brglm2::bracl()}}).
}
\note{
For the definition of complete and quasi-complete separation, see
Albert and Anderson (1984). Kosmidis and Firth (2021) prove that
the reduced-bias estimator that results by the penalization of the
logistic regression log-likelihood by Jeffreys prior takes always
finite values, even when some of the maximum likelihood estimates
are infinite. The reduced-bias estimates can be computed using the
\pkg{brglm2} R package.
}
\examples{

# endometrial data from Heinze \& Schemper (2002) (see ?endometrial)
data("endometrial", package = "detectseparation")
endometrial_ml <- glm(HG ~ NV + PI + EH, data = endometrial,
                      family = binomial("probit"))
# clearly the maximum likelihood estimate for the coefficient of
# NV is infinite
(estimates <- check_infinite_estimates(endometrial_ml))
plot(estimates)


\donttest{
# Aligator data (Agresti, 2002, Table~7.1)
if (requireNamespace("brglm2", quietly = TRUE)) {
    data("alligators", package = "brglm2")
    all_ml <- brglm2::brmultinom(foodchoice ~ size + lake , weights = round(freq/3),
                         data = alligators, type = "ML", ref = 1)
    # Clearly some estimated standard errors diverge as the number of
    # Fisher scoring iterations increases
    plot(check_infinite_estimates(all_ml))
    # Bias reduction the brglm2 R packages can be used to get finite estimates
    all_br <- brglm2::brmultinom(foodchoice ~ size + lake , weights = round(freq/3),
                         data = alligators, ref = 1)
    plot(check_infinite_estimates(all_br))
}
}
}
\references{
Lesaffre, E., & Albert, A. (1989). Partial Separation in Logistic
Discrimination. \emph{Journal of the Royal Statistical Society. Series B
(Methodological)}, \strong{51}, 109-116

Kosmidis I. and Firth D. (2021). Jeffreys-prior penalty, finiteness
and shrinkage in binomial-response generalized linear
models. \emph{Biometrika}, \strong{108}, 71–82
}
\seealso{
\code{\link[nnet:multinom]{nnet::multinom()}},
\code{\link[=detect_separation]{detect_separation()}},
\code{\link[brglm2:brmultinom]{brglm2::brmultinom()}},
\code{\link[brglm2:bracl]{brglm2::bracl()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/detectseparation-package.R, R/zzz_conventions.R
\name{check_infinite_estimates}
\alias{check_infinite_estimates}
\alias{checkInfiniteEstimates}
\title{Generic method for checking for infinite estimates}
\usage{
check_infinite_estimates(object, ...)

checkInfiniteEstimates(object, ...)
}
\arguments{
\item{object}{a fitted model object (e.g. the result of a
\code{\link[=glm]{glm()}} call).}

\item{...}{other options to be passed to the method.}
}
\description{
Generic method for checking for infinite estimates
}
\seealso{
check_infinite_estimates.glm
}
